using System;
using System.Data;
using System.Web.UI.WebControls;
using System.Web.UI;

using CMS.CMSHelper;
using CMS.FileManager;
using CMS.GlobalHelper;
using CMS.TreeEngine;
using CMS.WorkflowEngine;
using CMS.FormControls;
using CMS.DataEngine;
using CMS.SiteProvider;

using TreeNode = CMS.TreeEngine.TreeNode;

public partial class CMSAdminControls_Attachments_DocumentAttachmentsList : AttachmentsControl
{
    #region "Constants"

    private const int ATTACHMENT_NAME_LIMIT = 90;

    #endregion


    #region "Variables"

    private TreeProvider mTreeProvider = null;
    private TreeNode mNode = null;
    private AttachmentManager mAttachmentManager = null;
    private bool? mUsesWorkflow = null;
    private bool? mAutoCheck = null;
    private string mInnerDivClass = "NewAttachment";
    private string mInnerLoadingDivClass = "NewAttachmentLoading";
    private bool mShowTooltip = true;
    private string mSiteName = null;

    #endregion


    #region "Properties"

    /// <summary>
    /// Site name
    /// </summary>
    private string SiteName
    {
        get
        {
            if (mSiteName == null)
            {
                mSiteName = CMSContext.CurrentSiteName;
            }
            return mSiteName;
        }
        set
        {
            mSiteName = value;
        }
    }


    /// <summary>
    /// CSS class of the new attachment link
    /// </summary>
    public string InnerDivClass
    {
        get
        {
            return mInnerDivClass;
        }
        set
        {
            mInnerDivClass = value;
            if (lblDisabled != null)
            {
                lblDisabled.CssClass = value + "Disabled";
            }
        }
    }


    /// <summary>
    /// CSS class of the new attachment loading element
    /// </summary>
    public string InnerLoadingDivClass
    {
        get
        {
            return mInnerLoadingDivClass;
        }
        set
        {
            mInnerLoadingDivClass = value;
        }
    }


    /// <summary>
    /// Info label
    /// </summary>
    public Label InfoLabel
    {
        get
        {
            return lblInfo;
        }
    }


    /// <summary>
    /// Indicates whether grouped attachments should be displayed
    /// </summary>
    public override Guid GroupGUID
    {
        get
        {
            return base.GroupGUID;
        }
        set
        {
            base.GroupGUID = value;
            if ((dsAttachments != null) && (newAttachmentElem != null))
            {
                dsAttachments.AttachmentGroupGUID = value;
                newAttachmentElem.AttachmentGroupGUID = value;
            }
        }
    }


    /// <summary>
    /// Gets or sets tree provider
    /// </summary>
    public TreeProvider TreeProvider
    {
        get
        {
            if (mTreeProvider == null)
            {
                mTreeProvider = new TreeProvider(CMSContext.CurrentUser, ConnectionHelper.GetConnection());
            }
            return mTreeProvider;
        }
        set
        {
            mTreeProvider = value;
        }
    }


    /// <summary>
    /// Gets tree node
    /// </summary>
    public TreeNode Node
    {
        get
        {
            if (mNode == null)
            {
                mNode = TreeProvider.SelectSingleDocument(DocumentID);
            }
            return mNode;
        }
    }


    /// <summary>
    /// Gets attachment manager
    /// </summary>
    private AttachmentManager AttachmentManager
    {
        get
        {
            if (mAttachmentManager == null)
            {
                mAttachmentManager = new AttachmentManager(TreeProvider.Connection);
            }
            return mAttachmentManager;
        }
    }


    /// <summary>
    /// Indicates if worflow is used
    /// </summary>
    private bool UsesWorkflow
    {
        get
        {
            if (mUsesWorkflow == null)
            {
                if (Node != null)
                {
                    WorkflowManager workflowMan = new WorkflowManager(TreeProvider);

                    // Check if the document uses workflow
                    mUsesWorkflow = (workflowMan.GetNodeWorkflow(Node) != null);
                }
                else
                {
                    mUsesWorkflow = false;
                }
            }

            return mUsesWorkflow.Value;
        }
    }


    /// <summary>
    /// Indicates if automatic check-in/check-out should be used
    /// </summary>
    private bool AutoCheck
    {
        get
        {
            if (mAutoCheck == null)
            {
                mAutoCheck = !VersionManager.UseCheckInCheckOut(SiteName);
            }

            return mAutoCheck.Value;
        }
    }


    /// <summary>
    /// Indicates if user is allowed to change the order of the attachments
    /// </summary>
    public override bool AllowChangeOrder
    {
        get
        {
            return base.AllowChangeOrder;
        }
        set
        {
            base.AllowChangeOrder = value;
        }
    }


    /// <summary>
    /// Indicates if paging is allowed
    /// </summary>
    public override bool AllowPaging
    {
        get
        {
            return base.AllowPaging;
        }
        set
        {
            base.AllowPaging = value;
            if (gridAttachments != null)
            {
                gridAttachments.GridView.AllowPaging = value;
            }
        }
    }


    /// <summary>
    /// Defines size of the page for paging
    /// </summary>
    public override int PageSize
    {
        get
        {
            return base.PageSize;
        }
        set
        {
            base.PageSize = value;
            if (gridAttachments != null)
            {
                gridAttachments.GridView.PageSize = value;
            }
        }
    }


    /// <summary>
    /// Width of the attachment
    /// </summary>
    public override int ResizeToWidth
    {
        get
        {
            return base.ResizeToWidth;
        }
        set
        {
            base.ResizeToWidth = value;
            if (newAttachmentElem != null)
            {
                newAttachmentElem.ResizeToWidth = value;
            }
        }
    }


    /// <summary>
    /// Height of the attachment
    /// </summary>
    public override int ResizeToHeight
    {
        get
        {
            return base.ResizeToHeight;
        }
        set
        {
            base.ResizeToHeight = value;
            if (newAttachmentElem != null)
            {
                newAttachmentElem.ResizeToHeight = value;
            }
        }
    }


    /// <summary>
    /// Maximal side size of the attachment
    /// </summary>
    public override int ResizeToMaxSideSize
    {
        get
        {
            return base.ResizeToMaxSideSize;
        }
        set
        {
            base.ResizeToMaxSideSize = value;
            if (newAttachmentElem != null)
            {
                newAttachmentElem.ResizeToMaxSideSize = value;
            }
        }
    }


    /// <summary>
    /// List of allowed extensions
    /// </summary>
    public override string AllowedExtensions
    {
        get
        {
            return base.AllowedExtensions;
        }
        set
        {
            base.AllowedExtensions = value;
            if (newAttachmentElem != null)
            {
                newAttachmentElem.AllowedExtensions = value;
            }
        }
    }


    /// <summary>
    /// Specifies the document for which the attachments should be displayed
    /// </summary>
    public override int DocumentID
    {
        get
        {
            return base.DocumentID;
        }
        set
        {
            base.DocumentID = value;
            if (newAttachmentElem != null)
            {
                newAttachmentElem.DocumentID = value;
            }
        }
    }


    /// <summary>
    /// Specifies the version of the document (optional)
    /// </summary>
    public override int VersionHistoryID
    {
        get
        {
            return base.VersionHistoryID;
        }
        set
        {
            base.VersionHistoryID = value;
            if (dsAttachments != null)
            {
                dsAttachments.DocumentVersionHistoryID = value;
            }
        }
    }


    /// <summary>
    /// Defines the form GUID; indicates that the temporary attachment will be handled
    /// </summary>
    public override Guid FormGUID
    {
        get
        {
            return base.FormGUID;
        }
        set
        {
            base.FormGUID = value;
            if ((dsAttachments != null) && (newAttachmentElem != null))
            {
                dsAttachments.AttachmentFormGUID = value;
                newAttachmentElem.FormGUID = value;
            }
        }
    }


    /// <summary>
    /// Indicates if the control is enabled
    /// </summary>
    public override bool Enabled
    {
        get
        {
            return base.Enabled;
        }
        set
        {
            base.Enabled = value;
        }
    }


    /// <summary>
    /// Indicates if toltips should be shown
    /// </summary>
    public bool ShowTooltip
    {
        get
        {
            return mShowTooltip;
        }
        set
        {
            mShowTooltip = value;
        }
    }

    #endregion


    #region "Page events"

    protected void Page_Load(object sender, EventArgs e)
    {
        // Register script for tooltips
        if (ShowTooltip)
        {
            ScriptHelper.RegisterTooltip(Page);
        }

        // Ensure info message
        if ((Request["__EVENTTARGET"] == hdnPostback.ClientID) || Request["__EVENTTARGET"] == hdnFullPostback.ClientID)
        {
            string action = Request["__EVENTARGUMENT"].ToString();

            switch (action)
            {
                case "insert":
                    lblInfo.Text = ResHelper.GetString("attach.inserted");
                    break;

                case "update":
                    lblInfo.Text = ResHelper.GetString("attach.updated");
                    break;

                case "delete":
                    lblInfo.Text = ResHelper.GetString("attach.deleted");
                    break;

                default:
                    if (action != "")
                    {
                        UpdateEditParameters(action);
                    }
                    break;
            }
        }

        // Register jQuery script for thumbnails updating
        ScriptHelper.RegisterJQuery(this.Page);

        string refreshScript = @"function RefreshUpdatePanel(hiddenFieldID, action) {
                        var hiddenField = document.getElementById(hiddenFieldID);
                        if (hiddenField) {
                            __doPostBack(hiddenFieldID, action);
                        }
                    }

                    function FullRefresh(hiddenFieldID, action) {
                        if(PassiveRefresh != null)
                        {
                            PassiveRefresh();
                        }

                        var hiddenField = document.getElementById(hiddenFieldID);
                        if (hiddenField) {
                            __doPostBack(hiddenFieldID, action);
                        }
                    }

                    function FullPageRefresh(guid) {
                        if(PassiveRefresh != null)
                        {
                            PassiveRefresh();
                        }

                        var hiddenField = document.getElementById('" + hdnFullPostback.ClientID + "');" +
                        @"if (hiddenField) {
                            __doPostBack('" + hdnFullPostback.ClientID + "', 'refresh|' + guid);" +
                        @"}
                    }";

        Page.ClientScript.RegisterClientScriptBlock(GetType(), "RefreshUpdatePanel", ScriptHelper.GetScript(refreshScript));

        // Initialize refresh script for update panel
        string initRefreshScript =
            "function InitRefresh_" + ClientID + "(msg, fullRefresh, action)\n" +
            "{\n" +
            "   if((msg != null) && (msg != \"\")){ alert(msg); action='error'; }\n" +
            "   if(fullRefresh){ FullRefresh('" + hdnFullPostback.ClientID + "', action); }\n" +
            "   else { RefreshUpdatePanel('" + hdnPostback.ClientID + "', action); }\n" +
            "}\n";
        ltlScript.Text = ScriptHelper.GetScript(initRefreshScript);

        // Register dialog script
        Page.ClientScript.RegisterClientScriptBlock(typeof(string), ScriptHelper.DIALOG_SCRIPT_KEY, ScriptHelper.DialogScript);

        // Dialog for editing image
        Page.ClientScript.RegisterClientScriptBlock(typeof(string), "EditImage_" + ClientID,
            ScriptHelper.GetScript("function EditImage_" + ClientID + "(attachmentGUID, formGUID, versionHistoryID, parentId) { " +
                "var form = ''; \n" +
                "if(formGUID != ''){ form = '&formguid=' + formGUID + '&parentid=' + parentId;}" +
                ((Node != null) ? "else{form = '&siteid=' + " + Node.NodeSiteID + ";}" : "") +
                "modalDialog('" + ResolveUrl((this.IsLiveSite ? "~/CMSFormControls/LiveSelectors/ImageEditor.aspx" : "~/CMSDesk/Content/ImageEditor.aspx") + "?attachmentGUID=' + attachmentGUID + '&versionHistoryID=' + versionHistoryID + form + '&clientid=" + ClientID + "'") + ", 'imageEditorDialog', 900, 600); " +
                "return false; } "));

        // Initialize deletion confirmation
        string deleteConfirmation = "function DeleteConfirmation(){var conf = confirm('" +
                                    ResHelper.GetString("attach.deleteconfirmation") + "'); return conf;}";

        Page.ClientScript.RegisterClientScriptBlock(GetType(), "DeleteConfirmation", ScriptHelper.GetScript(deleteConfirmation));

        // Grid initialization
        gridAttachments.OnExternalDataBound += GridDocsOnExternalDataBound;
        gridAttachments.OnAction += GridAttachmentsOnAction;
        gridAttachments.ImageDirectoryPath = UrlHelper.ImagesDirectory + "/CMSAdminControls/UniGrid/";
        pnlGrid.Attributes.Add("style", "margin-top: 5px;");

        // Load data
        ReloadData();
    }


    /// <summary>
    /// Updates parameters used by Edit button when displaying image editor.
    /// </summary>
    /// <param name="action"></param>
    private void UpdateEditParameters(string action)
    {
        if (this.ShowTooltip)
        {
            // Try to get attachment Guid
            Guid attGuid = ValidationHelper.GetGuid(action, Guid.Empty);
            if (attGuid != null)
            {
                // Get attachment
                AttachmentInfo attInfo = AttachmentManager.GetAttachmentInfo(attGuid, SiteName);
                if (attInfo != null)
                {
                    // Get attachment URL
                    string attachmentUrl = null;
                    if (Node != null)
                    {
                        attachmentUrl = ResolveUrl(TreePathUtils.GetAttachmentUrl(attGuid, UrlHelper.GetSafeFileName(attInfo.AttachmentName, CMSContext.CurrentSiteName), VersionHistoryID, "getattachment"));
                    }
                    else
                    {
                        attachmentUrl = ResolveUrl(DocumentHelper.GetAttachmentUrl(attGuid, VersionHistoryID));
                    }
                    attachmentUrl = UrlHelper.UpdateParameterInUrl(attachmentUrl, "chset", Guid.NewGuid().ToString());

                    // Ensure correct URL
                    if (SiteName != CMSContext.CurrentSiteName)
                    {
                        attachmentUrl = UrlHelper.AddParameterToUrl(attachmentUrl, "sitename", SiteName);
                    }

                    // Ensure max width
                    int attachmentWidth = attInfo.AttachmentImageWidth;
                    if (attachmentWidth > 300)
                    {
                        attachmentWidth = 300;
                    }

                    // Generate new tooltip command
                    string newToolTip = "Tip('<div style=\\'width:" + attachmentWidth + "px; text-align:center;\\'><img src=\\'" +
                        UrlHelper.AddParameterToUrl(attachmentUrl, "width", "300") + "\\' alt=\\'" + attInfo.AttachmentName + "\\' /></div>')";

                    // Get update script
                    string updateScript = "$(\"#" + attGuid + "\").attr('onmouseover', '').unbind('mouseover').mouseover(function(){ " + newToolTip + " });";

                    // Execute update
                    ScriptManager.RegisterStartupScript(this.Page, typeof(Page), "AttachmentUpdateEdit", ScriptHelper.GetScript(updateScript), false);
                }
            }
        }
    }


    protected override void OnPreRender(EventArgs e)
    {
        base.OnPreRender(e);

        lblError.Visible = (lblError.Text != "");
        lblInfo.Visible = (lblInfo.Text != "");

        // Ensure uplaoder button
        plcUploader.Visible = Enabled;
        plcUploaderDisabled.Visible = !Enabled;

        // Hide actions
        gridAttachments.GridView.Columns[0].Visible = !HideActions;
        gridAttachments.GridView.Columns[1].Visible = !HideActions;
        newAttachmentElem.Visible = !HideActions && Enabled;
        plcUploaderDisabled.Visible = !HideActions && !Enabled;

        // Ensure correct layout
        bool gridHasData = !DataHelper.DataSourceIsEmpty(gridAttachments.DataSource);
        this.Visible = gridHasData || !HideActions;
        pnlGrid.Visible = gridHasData;
    }

    #endregion


    #region "Public methods"

    /// <summary>
    /// Indicates if the control contains some data
    /// </summary>
    public override bool HasData()
    {
        return !DataHelper.DataSourceIsEmpty(dsAttachments.DataSource);
    }

    #endregion


    #region "Private & protected methods"

    public void ReloadData(bool forceReload)
    {
        if (StopProcessing)
        {
            // Do nothing
        }
        else
        {
            if (Node != null)
            {
                dsAttachments.Path = Node.NodeAliasPath;
                dsAttachments.CultureCode = CMSContext.PreferredCultureCode;

                SiteInfo si = SiteInfoProvider.GetSiteInfo(Node.NodeSiteID);
                SiteName = si.SiteName;
                dsAttachments.SiteName = SiteName;
            }

            // Versioned attachments
            bool attachmentHistory = (VersionHistoryID != 0) && (UsesWorkflow || (Node == null));

            dsAttachments.DocumentVersionHistoryID = VersionHistoryID;
            dsAttachments.AttachmentGroupGUID = GroupGUID;
            dsAttachments.AttachmentFormGUID = FormGUID;
            dsAttachments.OrderBy = "AttachmentOrder, AttachmentName, " + (attachmentHistory ? "AttachmentHistoryID" : "AttachmentID");

            // Force reload datasource
            if (forceReload)
            {
                dsAttachments.DataSource = null;
                dsAttachments.DataBind();
            }

            // Initialize button for adding attachments
            newAttachmentElem.ImageUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSAdminControls/Attachments/upload_new.gif");
            newAttachmentElem.ImageWidth = 200;
            newAttachmentElem.ImageHeight = 16;
            newAttachmentElem.DocumentID = DocumentID;
            newAttachmentElem.NodeParentNodeID = NodeParentNodeID;
            newAttachmentElem.NodeClassName = NodeClassName;
            newAttachmentElem.ResizeToWidth = ResizeToWidth;
            newAttachmentElem.ResizeToHeight = ResizeToHeight;
            newAttachmentElem.ResizeToMaxSideSize = ResizeToMaxSideSize;
            newAttachmentElem.AttachmentGroupGUID = GroupGUID;
            newAttachmentElem.FormGUID = FormGUID;
            newAttachmentElem.AllowedExtensions = AllowedExtensions;
            newAttachmentElem.ParentElemID = ClientID;
            newAttachmentElem.ForceLoad = true;
            newAttachmentElem.InnerDivHtml = ResHelper.GetString("attach.newattachment");
            newAttachmentElem.InnerDivClass = InnerDivClass;
            newAttachmentElem.InnerLoadingDivHtml = ResHelper.GetString("attach.loading");
            newAttachmentElem.InnerLoadingDivClass = InnerLoadingDivClass;
            newAttachmentElem.IsLiveSite = IsLiveSite;
            newAttachmentElem.CheckPermissions = CheckPermissions;
            lblDisabled.CssClass = InnerDivClass + "Disabled";

            imgDisabled.ImageUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSAdminControls/Attachments/upload_newdisabled.gif");

            // Ensure right column name (for attachments under workflow)
            if (!DataHelper.DataSourceIsEmpty(dsAttachments.DataSource))
            {
                DataSet ds = (DataSet)dsAttachments.DataSource;
                if (ds != null)
                {
                    DataTable dt = (ds).Tables[0];
                    if (!dt.Columns.Contains("AttachmentFormGUID"))
                    {
                        dt.Columns.Add("AttachmentFormGUID");
                    }
                }
            }

            // Ensure right row order
            gridAttachments.OrderBy = "AttachmentOrder, AttachmentName, AttachmentID";
            // Bind UniGrid to DataSource
            gridAttachments.GridView.PageSize = PageSize;
            gridAttachments.GridView.AllowPaging = AllowPaging;
            gridAttachments.GridView.AllowSorting = false;
            gridAttachments.DataSource = dsAttachments.DataSource;
            gridAttachments.ReloadData();
        }
    }


    /// <summary>
    /// UniGrid action buttons event handler.
    /// </summary>
    protected void GridAttachmentsOnAction(string actionName, object actionArgument)
    {
        if (Enabled && !HideActions)
        {
            // Check the permissions
            #region "Check permissions"

            if (CheckPermissions)
            {
                if (FormGUID != Guid.Empty)
                {
                    if (!RaiseOnCheckPermissions("Create", this))
                    {
                        if (!CMSContext.CurrentUser.IsAuthorizedToCreateNewDocument(NodeParentNodeID, NodeClassName))
                        {
                            lblError.Text = ResHelper.GetString("attach.actiondenied");
                            return;
                        }
                    }
                }
                else
                {
                    if (!RaiseOnCheckPermissions("Modify", this))
                    {
                        if (CMSContext.CurrentUser.IsAuthorizedPerDocument(Node, NodePermissionsEnum.Modify) == AuthorizationResultEnum.Denied)
                        {
                            lblError.Text = ResHelper.GetString("attach.actiondenied");
                            return;
                        }
                    }
                }
            }

            #endregion

            int attachmentId = 0;
            Guid attachmentGuid = Guid.Empty;

            // Get action argument (Guid or int)
            if (ValidationHelper.IsGuid(actionArgument))
            {
                attachmentGuid = ValidationHelper.GetGuid(actionArgument, Guid.Empty);
            }
            if (ValidationHelper.IsInteger(actionArgument))
            {
                attachmentId = ValidationHelper.GetInteger(actionArgument, 0);
            }

            // Process proper action
            switch (actionName.ToLower())
            {
                case "moveup":
                    if (attachmentId != 0)
                    {
                        // Move attachment up
                        if (FormGUID == Guid.Empty)
                        {
                            DocumentHelper.MoveAttachmentUp(attachmentId, Node);

                            // Log synchronization task if not under workflow
                            if (!UsesWorkflow)
                            {
                                DocumentHelper.LogSynchronization(Node, CMS.Staging.TaskTypeEnum.UpdateDocument, TreeProvider);
                            }
                        }
                        else
                        {
                            AttachmentManager.MoveTemporaryAttachmentUp(attachmentId);
                        }
                    }
                    break;

                case "movedown":
                    if (attachmentId != 0)
                    {
                        // Move attachment down
                        if (FormGUID == Guid.Empty)
                        {
                            DocumentHelper.MoveAttachmentDown(attachmentId, Node);

                            // Log synchronization task if not under workflow
                            if (!UsesWorkflow)
                            {
                                DocumentHelper.LogSynchronization(Node, CMS.Staging.TaskTypeEnum.UpdateDocument, TreeProvider);
                            }
                        }
                        else
                        {
                            AttachmentManager.MoveTemporaryAttachmentDown(attachmentId);
                        }
                    }
                    break;

                case "delete":
                    if (attachmentGuid != Guid.Empty)
                    {
                        // Delete attachment
                        if (FormGUID == Guid.Empty)
                        {
                            // Ensure automatic check-in/ check-out
                            VersionManager vm = null;

                            // Check out the document
                            if (AutoCheck && UsesWorkflow)
                            {
                                vm = new VersionManager(TreeProvider);
                                vm.CheckOut(Node);
                            }

                            DocumentHelper.DeleteAttachment(Node, attachmentGuid, TreeProvider);

                            // Ensure full page refresh
                            if (UsesWorkflow && AutoCheck)
                            {
                                ScriptManager.RegisterStartupScript(this.Page, typeof(Page), "deleteRefresh", "InitRefresh_" + ClientID + "('', true, 'delete');", true);
                            }

                            // Check in the document
                            if (AutoCheck && UsesWorkflow)
                            {
                                if (vm != null)
                                {
                                    vm.CheckIn(Node, null, null);
                                    VersionHistoryID = ValidationHelper.GetInteger(Node.GetValue("DocumentCheckedOutVersionHistoryID"), 0);
                                }
                            }

                            // Log synchronization task if not under workflow
                            if (!UsesWorkflow)
                            {
                                DocumentHelper.LogSynchronization(Node, CMS.Staging.TaskTypeEnum.UpdateDocument, TreeProvider);
                            }
                        }
                        else
                        {
                            AttachmentManager.DeleteTemporaryAttachment(attachmentGuid, CMSContext.CurrentSiteName);
                        }

                        lblInfo.Text = ResHelper.GetString("attach.deleted");
                    }
                    break;
            }
            // Force reload data
            ReloadData(true);
        }
    }


    /// <summary>
    /// Overloaded ReloadData
    /// </summary>
    public void ReloadData()
    {
        ReloadData(false);
    }


    /// <summary>
    /// UniGrid external data bound.
    /// </summary>
    protected object GridDocsOnExternalDataBound(object sender, string sourceName, object parameter)
    {
        switch (sourceName.ToLower())
        {
            case "update":
                PlaceHolder plcUpd = new PlaceHolder();
                plcUpd.ID = "plcUdateAction";

                // Add disabled image
                ImageButton imgUpdate = new ImageButton();
                imgUpdate.ID = "imgUpdate";
                imgUpdate.PreRender += new EventHandler(imgUpdate_PreRender);
                plcUpd.Controls.Add(imgUpdate);

                // Add update control
                // Dynamically load uploader control
                CMSAdminControls_Attachments_DirectFileUploader_DirectFileUploader dfuElem = Page.LoadControl("~/CMSAdminControls/Attachments/DirectFileUploader/DirectFileUploader.ascx") as CMSAdminControls_Attachments_DirectFileUploader_DirectFileUploader;

                // Set uploader's properties
                if (dfuElem != null)
                {
                    dfuElem.ID = "dfuElem" + DocumentID;
                    dfuElem.ControlGroup = "update";
                    dfuElem.AttachmentGUID = GetAttachmentGuid(sender as DataControlFieldCell);
                    dfuElem.PreRender += new EventHandler(dfuElem_PreRender);
                    plcUpd.Controls.Add(dfuElem);
                }
                return plcUpd;

            case "edit":
                // Get file extension
                string extension = ValidationHelper.GetString(((DataRowView)((GridViewRow)parameter).DataItem).Row["AttachmentExtension"], string.Empty).ToLower();
                Guid guid = ValidationHelper.GetGuid(((DataRowView)((GridViewRow)parameter).DataItem).Row["AttachmentGUID"], Guid.Empty);

                ImageButton img = (ImageButton)sender;
                img.AlternateText = extension + "|" + guid;
                img.PreRender += new EventHandler(img_PreRender);
                break;

            case "delete":
                ImageButton imgDelete = (ImageButton)sender;
                // Turn off validation
                imgDelete.CausesValidation = false;
                imgDelete.PreRender += new EventHandler(imgDelete_PreRender);
                // Explicitly initialize confirmation
                imgDelete.OnClientClick = "if(DeleteConfirmation() == false){return false;}";
                break;

            case "moveup":
                ImageButton imgUp = (ImageButton)sender;
                // Turn off validation
                imgUp.CausesValidation = false;
                imgUp.PreRender += new EventHandler(imgUp_PreRender);
                break;

            case "movedown":
                ImageButton imgDown = (ImageButton)sender;
                // Turn off validation
                imgDown.CausesValidation = false;
                imgDown.PreRender += new EventHandler(imgDown_PreRender);
                break;

            case "attachmentname":
                DataControlFieldCell cell = sender as DataControlFieldCell;

                // Get attachment GUID
                Guid attachmentGuid = GetAttachmentGuid(cell);

                // Get attachment extension
                string attachmentExt = GetAttachmentExtension(cell);
                bool isImage = ImageHelper.IsImage(attachmentExt);
                string iconUrl = ImageHelper.GetFileIconUrl(attachmentExt, "List");

                // Get link for attachment
                string attachmentUrl = null;
                string attName = parameter.ToString();
                if (Node != null)
                {
                    attachmentUrl = ResolveUrl(TreePathUtils.GetAttachmentUrl(attachmentGuid, UrlHelper.GetSafeFileName(attName, CMSContext.CurrentSiteName), VersionHistoryID, "getattachment"));
                }
                else
                {
                    attachmentUrl = ResolveUrl(DocumentHelper.GetAttachmentUrl(attachmentGuid, VersionHistoryID));
                }
                attachmentUrl = UrlHelper.UpdateParameterInUrl(attachmentUrl, "chset", Guid.NewGuid().ToString());

                // Ensure correct URL
                if (SiteName != CMSContext.CurrentSiteName)
                {
                    attachmentUrl = UrlHelper.AddParameterToUrl(attachmentUrl, "sitename", SiteName);
                }

                // Optionally trim attachment name
                string attachmentName = TextHelper.LimitLength(attName, ATTACHMENT_NAME_LIMIT, "...");

                string tooltip = null;
                if (ShowTooltip && isImage)
                {
                    int attachmentWidth = GetAttachmentWidth(cell);
                    if (attachmentWidth > 300)
                    {
                        attachmentWidth = 300;
                    }
                    tooltip = "onmouseout=\"UnTip()\" onmouseover=\"Tip('<div style=\\'width:" + attachmentWidth + "px;text-align:center;\\'><img src=\\'" + UrlHelper.AddParameterToUrl(attachmentUrl, "width", "300") + "\\' alt=\\'" + attachmentName + "\\' /></div>')\"";
                }

                string imageTag = "<img class=\"Icon\" src=\"" + iconUrl + "\" alt=\"" + attachmentName + "\" />";
                if (isImage)
                {
                    return "<a href=\"#\" onclick=\"javascript: window.open('" + attachmentUrl + "'); return false;\"><span id=\"" + attachmentGuid + "\" " + tooltip + ">" + imageTag + attachmentName + "</span></a>";
                }
                else
                {
                    return "<a href=\"" + attachmentUrl + "\">" + imageTag + "</a><a href=\"" + attachmentUrl + "\">" + attachmentName + "</a>";
                }

            case "attachmentsize":
                long size = ValidationHelper.GetLong(parameter, 0);
                return DataHelper.GetSizeString(size);
        }

        return parameter;
    }


    #region "Grid actions events"

    void dfuElem_PreRender(object sender, EventArgs e)
    {
        CMSAdminControls_Attachments_DirectFileUploader_DirectFileUploader dfuElem = (CMSAdminControls_Attachments_DirectFileUploader_DirectFileUploader)sender;
        if (Enabled)
        {
            dfuElem.ForceLoad = true;
            dfuElem.FormGUID = FormGUID;
            dfuElem.AttachmentGroupGUID = GroupGUID;
            dfuElem.DocumentID = DocumentID;
            dfuElem.NodeParentNodeID = NodeParentNodeID;
            dfuElem.NodeClassName = NodeClassName;
            dfuElem.ResizeToWidth = ResizeToWidth;
            dfuElem.ResizeToHeight = ResizeToHeight;
            dfuElem.ResizeToMaxSideSize = ResizeToMaxSideSize;
            dfuElem.AllowedExtensions = AllowedExtensions;
            dfuElem.ImageUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSAdminControls/Attachments/upload.gif");
            dfuElem.ImageHeight = 16;
            dfuElem.ImageWidth = 16;
            dfuElem.InsertMode = false;
            dfuElem.ParentElemID = ClientID;
            dfuElem.CheckPermissions = CheckPermissions;

        }
        else
        {
            dfuElem.Visible = false;
        }
    }


    void imgUpdate_PreRender(object sender, EventArgs e)
    {
        ImageButton imgUpdate = (ImageButton)sender;
        if (!Enabled)
        {
            imgUpdate.ImageUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSAdminControls/Attachments/uploaddisabled.gif");
            imgUpdate.Style.Add("cursor", "default");
            imgUpdate.Enabled = false;
        }
        else
        {
            imgUpdate.Visible = false;
        }
    }


    void img_PreRender(object sender, EventArgs e)
    {
        ImageButton img = (ImageButton)sender;

        if (CMSContext.CurrentUser.IsAuthenticated())
        {
            string[] args = img.AlternateText.Split('|');
            // If the file is not an image don't allow image editing
            if (!ImageHelper.IsSupportedByImageEditor(args[0]) || !Enabled)
            {
                // Disable edit icon in case that attachment is not an image
                img.ImageUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSAdminControls/UniGrid/editdisabled.gif");
                img.Enabled = false;
                img.Style.Add("cursor", "default");
            }
            else
            {
                string strForm = (FormGUID == Guid.Empty) ? "" : FormGUID.ToString();
                img.Attributes.Add("onclick", "EditImage_" + ClientID + "('" + args[1] + "', '" + strForm + "', '" + VersionHistoryID + "', " + NodeParentNodeID + ");return false;");
            }

            img.AlternateText = ResHelper.GetString("general.edit");
        }
        else
        {
            img.Visible = false;
        }
    }


    void imgDown_PreRender(object sender, EventArgs e)
    {
        ImageButton imgDown = (ImageButton)sender;
        if (!Enabled || !AllowChangeOrder)
        {
            // Disable move down icon in case that editing is not allowed
            imgDown.ImageUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSAdminControls/UniGrid/movedowndisabled.gif");
            imgDown.Enabled = false;
            imgDown.Style.Add("cursor", "default");
        }
    }


    void imgUp_PreRender(object sender, EventArgs e)
    {
        ImageButton imgUp = (ImageButton)sender;
        if (!Enabled || !AllowChangeOrder)
        {
            // Disable move up icon in case that editing is not allowed
            imgUp.ImageUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSAdminControls/UniGrid/moveupdisabled.gif");
            imgUp.Enabled = false;
            imgUp.Style.Add("cursor", "default");
        }
    }


    void imgDelete_PreRender(object sender, EventArgs e)
    {
        ImageButton imgDelete = (ImageButton)sender;
        if (!Enabled)
        {
            // Disable delete icon in case that editing is not allowed
            imgDelete.ImageUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSAdminControls/UniGrid/deletedisabled.gif");
            imgDelete.Enabled = false;
            imgDelete.Style.Add("cursor", "default");
        }
    }

    #endregion


    /// <summary>
    /// Gets GUID value from DataControlFieldCell
    /// </summary>
    /// <param name="dcf">Row cell</param>
    /// <returns>GUID of attachment</returns>
    protected Guid GetAttachmentGuid(DataControlFieldCell dcf)
    {
        // Get GUID of attachment
        return ValidationHelper.GetGuid(GetAttachmentProperty(dcf, "AttachmentGUID"), Guid.Empty);
    }


    /// <summary>
    /// Gets width value from DataControlFieldCell
    /// </summary>
    /// <param name="dcf">Row cell</param>
    /// <returns>Width of attachment</returns>
    protected int GetAttachmentWidth(DataControlFieldCell dcf)
    {
        // Get width of attachment
        return ValidationHelper.GetInteger(GetAttachmentProperty(dcf, "AttachmentImageWidth"), 0);
    }


    /// <summary>
    /// Gets extension value from DataControlFieldCell
    /// </summary>
    /// <param name="dcf">Row cell</param>
    /// <returns>Extension of attachment</returns>
    protected string GetAttachmentExtension(DataControlFieldCell dcf)
    {
        // Get extension of attachment
        return ValidationHelper.GetString(GetAttachmentProperty(dcf, "AttachmentExtension"), "");
    }


    protected object GetAttachmentProperty(DataControlFieldCell dcf, string property)
    {
        // Get datarowview
        DataRowView drv = ((GridViewRow)(dcf).Parent).DataItem as DataRowView;
        if (drv != null)
        {
            // Get GUID of attachment
            return drv.Row[property];
        }
        return null;
    }

    #endregion
}
