﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Collections;
using System.Data;
using System.IO;
using System.Text;

using CMS.UIControls;
using CMS.GlobalHelper;
using CMS.SettingsProvider;
using CMS.ExtendedControls;
using CMS.SiteProvider;
using CMS.CMSHelper;
using CMS.TreeEngine;
using CMS.WorkflowEngine;
using CMS.FileManager;
using CMS.DataEngine;

using TreeNode = CMS.TreeEngine.TreeNode;


public partial class CMSAdminControls_Dialogs_LinkMediaSelector_LinkMediaSelector : LinkMediaSelector
{
    #region "Private variables"

    // Content variables
    private int mSiteID = 0;
    private int mNodeID = 0;

    private bool mIsAction = false;

    private TreeNode mTreeNodeObj = null;
    private AttachmentManager mAttachmentManager = null;
    private AttachmentInfo mCurrentAttachment = null;

    #endregion


    #region "Private properties"

    /// <summary>
    /// Gets or sets last searched value.
    /// </summary>
    private string LastSearchedValue
    {
        get
        {
            return this.hdnLastSearchedValue.Value;
        }
        set
        {
            this.hdnLastSearchedValue.Value = value;
        }
    }


    /// <summary>
    /// Gets current action name.
    /// </summary>
    private string CurrentAction
    {
        get
        {
            return this.hdnAction.Value.ToLower().Trim();
        }
        set
        {
            this.hdnAction.Value = value;
        }
    }


    /// <summary>
    /// Gets current action argument value.
    /// </summary>
    private string CurrentArgument
    {
        get
        {
            return this.hdnArgument.Value;
        }
    }


    /// <summary>
    /// Returns current properties (according to OutputFormat).
    /// </summary>
    protected override ItemProperties Properties
    {
        get
        {
            switch (this.Config.OutputFormat)
            {
                case OutputFormatEnum.HTMLMedia:
                    return this.htmlMediaProp;

                case OutputFormatEnum.HTMLLink:
                    return this.htmlLinkProp;

                case OutputFormatEnum.BBMedia:
                    return this.bbMediaProp;

                case OutputFormatEnum.BBLink:
                    return this.bbLinkProp;

                default:
                    if (this.Config.OutputFormat == OutputFormatEnum.NodeGUID)
                    {
                        return this.nodeGuidProp;
                    }
                    return this.urlProp;
            }
        }
    }


    /// <summary>
    /// Update panel where properties control resides.
    /// </summary>
    protected override UpdatePanel PropertiesUpdatePanel
    {
        get
        {
            return this.pnlUpdateProperties;
        }
    }


    /// <summary>
    /// Gets ID of the site files are being displayed for.
    /// </summary>
    private int SiteID
    {
        get
        {
            if (this.mSiteID == 0)
            {
                this.mSiteID = this.siteSelector.SiteID;
            }
            return this.mSiteID;
        }
        set
        {
            this.mSiteID = value;
        }
    }


    /// <summary>
    /// Gets name of the site files are being displayed for.
    /// </summary>
    private string SiteName
    {
        get
        {
            return this.siteSelector.SiteName;
        }
    }


    /// <summary>
    /// Gets or sets ID of the node selected in the content tree.
    /// </summary>
    private int NodeID
    {
        get
        {
            if (this.mNodeID == 0)
            {
                this.mNodeID = ValidationHelper.GetInteger(this.hdnLastNodeSlected.Value, 0);
            }
            return this.mNodeID;
        }
        set
        {
            this.mNodeID = value;
            this.hdnLastNodeSlected.Value = value.ToString();
            this.mTreeNodeObj = null;
        }
    }


    /// <summary>
    /// Maximum number of tree nodes displayed within the tree.
    /// </summary>
    private int MaxTreeNodes
    {
        get
        {
            return SettingsKeyProvider.GetIntValue(this.SiteName + ".CMSMaxTreeNodes");
        }
    }


    /// <summary>
    /// Indicates whether the attachments are temporary.
    /// </summary>
    private bool AttachmentsAreTemporary
    {
        get
        {
            return ((this.Config.AttachmentFormGUID != Guid.Empty) && (this.Config.AttachmentDocumentID == 0));
        }
    }


    /// <summary>
    /// Gets the node attachments are related to.
    /// </summary>
    private TreeNode TreeNodeObj
    {
        get
        {
            if (this.mTreeNodeObj == null)
            {
                // Content tab
                if (this.SourceType == MediaSourceEnum.Content)
                {
                    if (this.NodeID > 0)
                    {
                        TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
                        this.mTreeNodeObj = DocumentHelper.GetDocument(TreeProvider.ALL_SITES, null, TreeProvider.ALL_CULTURES, true, null, "NodeID = " + this.NodeID, null, -1, false, null, tree);
                    }
                }
                // Attachments tab
                else if (!this.AttachmentsAreTemporary)
                {
                    TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
                    this.mTreeNodeObj = DocumentHelper.GetDocument(this.Config.AttachmentDocumentID, tree);
                }

                this.mediaView.TreeNodeObj = this.mTreeNodeObj;
            }
            return this.mTreeNodeObj;
        }
        set
        {
            this.mTreeNodeObj = value;
        }
    }


    /// <summary>
    /// Gets manager object for manipulating with attachments.
    /// </summary>
    private AttachmentManager AttachmentManager
    {
        get
        {
            if (this.mAttachmentManager == null)
            {
                this.mAttachmentManager = new AttachmentManager();
            }
            return this.mAttachmentManager;
        }
    }


    /// <summary>
    /// Gets or sets GUID of the recently selected attachment.
    /// </summary>
    private Guid LastAttachmentGuid
    {
        get
        {
            return ValidationHelper.GetGuid(ViewState["LastAttachmentGuid"], Guid.Empty);
        }
        set
        {
            ViewState["LastAttachmentGuid"] = value;
        }
    }


    /// <summary>
    /// Currently selected item.
    /// </summary>
    private AttachmentInfo CurrentAttachmentInfo
    {
        get
        {
            return this.mCurrentAttachment;
        }
        set
        {
            this.mCurrentAttachment = value;
        }
    }


    /// <summary>
    /// Indicates whether the asynchronous postback occurs on the page.
    /// </summary>
    private bool IsAsyncPostback
    {
        get
        {
            return ScriptManager.GetCurrent(this.Page).IsInAsyncPostBack;
        }
    }


    /// <summary>
    /// Indicates whether the post back is result of some hidden action.
    /// </summary>
    private bool IsAction
    {
        get
        {
            return this.mIsAction;
        }
        set
        {
            this.mIsAction = value;
        }
    }


    /// <summary>
    /// Indicates whether the content tree is displaying more than max tree nodes.
    /// </summary>
    private bool IsDisplayMore
    {
        get
        {
            return this.mediaView.IsDisplayMore;
        }
        set
        {
            this.mediaView.IsDisplayMore = value;
        }
    }


    /// <summary>
    /// Gets or sets selected item to colorize.
    /// </summary>
    private Guid ItemToColorize
    {
        get
        {
            return ValidationHelper.GetGuid(ViewState["ItemToColorize"], Guid.Empty);
        }
        set
        {
            ViewState["ItemToColorize"] = value;
        }
    }


    /// <summary>
    /// Gets or sets ID of the node reflecting new root specified by starting path.
    /// </summary>
    private int StartingPathNodeID
    {
        get
        {
            return ValidationHelper.GetInteger(ViewState["StartingPathNodeID"], 0);
        }
        set
        {
            ViewState["StartingPathNodeID"] = value;
        }
    }


    /// <summary>
    /// Indicates if properties are displayed in full height mode
    /// </summary>
    private bool IsFullDisplay
    {
        get
        {
            return ValidationHelper.GetBoolean(ViewState["IsFullDisplay"], false);
        }
        set
        {
            ViewState["IsFullDisplay"] = value;
        }
    }

    #endregion


    protected override void OnInit(EventArgs e)
    {
        base.OnInit(e);

        this.menuElem.Config = this.Config;
        this.mediaView.Config = this.Config;

        // Get source type according URL parameters
        this.SourceType = CMSDialogHelper.GetMediaSource(QueryHelper.GetString("source", "attachments"));
        this.mediaView.OutputFormat = this.Config.OutputFormat;
    }


    protected override void OnPreRender(EventArgs e)
    {
        base.OnPreRender(e);

        // High-light item being edited
        if (this.ItemToColorize != Guid.Empty)
        {
            ColorizeRow(this.ItemToColorize.ToString());
        }

        // Display info on listing more content
        if (this.mediaView.ShowParentButton && (this.StartingPathNodeID != this.NodeID))
        {
            this.mediaView.ShowParentButton = this.IsDisplayMore;
        }
        else
        {
            this.mediaView.ShowParentButton = false;
        }
        if (this.IsDisplayMore && (this.TreeNodeObj != null))
        {
            string closeLink = "<span class=\"ListingClose\" style=\"cursor: pointer;\" onclick=\"SetAction('closelisting', ''); RaiseHiddenPostBack(); return false;\">" + ResHelper.GetString("general.close") + "</span>";
            string docNamePath = "<span class=\"ListingPath\">" + this.TreeNodeObj.DocumentNamePath + "</span>";

            string listingMsg = string.Format(ResHelper.GetString("dialogs.content.listingInfo"), docNamePath, closeLink);
            this.mediaView.DisplayListingInfo(listingMsg);
        }
    }


    protected void Page_Load(object sender, EventArgs e)
    {
        if (!this.StopProcessing)
        {
            CheckPermissions();

            SetupProperties();

            InitializeDialogs();

            SetupControls();

            EnsureLoadedData();
        }
        else
        {
            this.siteSelector.StopProcessing = true;
            this.Visible = false;
        }
    }


    #region "Inherited methods"

    /// <summary>
    /// Initializes its properties according to the URL parameters.
    /// </summary>
    public void InitFromQueryString()
    {
        switch (this.Config.OutputFormat)
        {
            case OutputFormatEnum.HTMLMedia:
                this.SelectableContent = SelectableContentEnum.OnlyMedia;
                break;

            case OutputFormatEnum.HTMLLink:
                this.SelectableContent = SelectableContentEnum.AllContent;
                break;

            case OutputFormatEnum.BBMedia:
                this.SelectableContent = SelectableContentEnum.OnlyImages;
                break;

            case OutputFormatEnum.BBLink:
                this.SelectableContent = SelectableContentEnum.AllContent;
                break;

            case OutputFormatEnum.URL:
            case OutputFormatEnum.NodeGUID:
                string content = QueryHelper.GetString("content", "");
                this.SelectableContent = CMSDialogHelper.GetSelectableContent(content);
                break;
        }
    }


    /// <summary>
    /// Returns selected item parameters as name-value collection.
    /// </summary>
    public void GetSelectedItem()
    {
        // Clear unused information from session
        ClearSelectedItemInfo();

        if (this.Properties.Validate())
        {
            // Store tab information in the user's dialogs configuration
            StoreDialogsConfiguration();

            // Get selected item information
            Hashtable properties = this.Properties.GetItemProperties();

            // Get JavaScript for inserting the item
            string script = GetInsertItem(properties);
            if (!string.IsNullOrEmpty(script))
            {
                ScriptManager.RegisterStartupScript(this.Page, typeof(Page), "insertItemScript", script, true);
            }
        }
        else
        {
            // Display error message
            this.pnlUpdateProperties.Update();
        }
    }

    #endregion


    #region "Dialog configuration"

    /// <summary>
    /// Stores current tab's configuration for the user.
    /// </summary>
    private void StoreDialogsConfiguration()
    {
        UserInfo ui = CMSContext.CurrentUser;

        // Store configuration based on the current tab
        switch (this.SourceType)
        {
            case MediaSourceEnum.Content:
                // Actualize configuration
                ui.UserSettings.UserDialogsConfiguration["content.sitename"] = this.SiteName;
                ui.UserSettings.UserDialogsConfiguration["content.path"] = GetContentPath(this.NodeID);
                ui.UserSettings.UserDialogsConfiguration["content.viewmode"] = CMSDialogHelper.GetDialogViewMode(this.menuElem.SelectedViewMode);
                break;

            case MediaSourceEnum.DocumentAttachments:
                // Actualize configuration
                ui.UserSettings.UserDialogsConfiguration["attachments.viewmode"] = CMSDialogHelper.GetDialogViewMode(this.menuElem.SelectedViewMode);
                break;
        }
        ui.UserSettings.UserDialogsConfiguration["selectedtab"] = CMSDialogHelper.GetMediaSource(this.SourceType);

        UserInfoProvider.SetUserInfo(ui);
    }


    /// <summary>
    /// Initializes dialogs according URL configuration, selected item or user configuration.
    /// </summary>
    private void InitializeDialogs()
    {
        if (!UrlHelper.IsPostback())
        {
            LoadDialogConfiguration();

            // Item is selected in the editor
            if (this.MediaSource != null)
            {
                LoadItemConfiguration();
            }
            else
            {
                LoadUserConfiguration();
            }
        }
    }


    /// <summary>
    /// Loads dialogs according configuration comming from the URL.
    /// </summary>
    private void LoadDialogConfiguration()
    {
        if (this.SourceType == MediaSourceEnum.Content)
        {
            // Initialize site selector
            this.siteSelector.StopProcessing = false;
            this.siteSelector.WhereCondition = GetSiteWhere();
            this.siteSelector.ReloadData();

            if (!string.IsNullOrEmpty(this.Config.ContentSelectedSite))
            {
                this.contentTree.SiteName = this.Config.ContentSelectedSite;
                this.siteSelector.SiteName = this.Config.ContentSelectedSite;
            }
            else
            {
                // Select default site
                string siteName = CMSContext.CurrentSiteName;

                this.contentTree.SiteName = siteName;
                this.siteSelector.SiteName = siteName;
            }

            this.pnlUpdateSelectors.Update();
        }
    }


    /// <summary>
    /// Gets WHERE condition for available sites according field configuration.
    /// </summary>
    private string GetSiteWhere()
    {
        string siteName = "";

        if (this.Config.ContentSites == AvailableSitesEnum.OnlySingleSite)
        {
            siteName = this.Config.ContentSelectedSite;
        }
        else if (this.Config.ContentSites == AvailableSitesEnum.OnlyCurrentSite)
        {
            siteName = CMSContext.CurrentSiteName;
        }

        if (siteName != "")
        {
            return "SiteName LIKE N'" + siteName + "'";
        }

        return siteName;
    }


    /// <summary>
    /// Loads selected item parameters into the selector.
    /// </summary>
    public void LoadItemConfiguration()
    {
        if (this.MediaSource != null)
        {
            this.IsItemLoaded = true;

            switch (this.MediaSource.SourceType)
            {
                case MediaSourceEnum.Content:
                    this.siteSelector.SiteID = this.MediaSource.SiteID;
                    this.contentTree.SiteName = this.SiteName;

                    // Try to select node in the tree
                    if (this.MediaSource.NodeID > 0)
                    {
                        this.NodeID = this.MediaSource.NodeID;
                        this.contentTree.NodeID = this.NodeID;
                    }
                    break;

                default:
                    break;
            }

            // Reload HTML properties
            if (this.Config.OutputFormat == OutputFormatEnum.HTMLMedia)
            {
                // Force media properties control to load selected item
                this.htmlMediaProp.ViewMode = this.MediaSource.MediaType;
            }

            // Display properties in full size
            if (this.SourceType == MediaSourceEnum.Content)
            {
                DisplayFull();
            }
        }

        // Load properties
        this.Properties.LoadItemProperties(this.Parameters);
        this.pnlUpdateProperties.Update();

        // Remember item to colorize
        this.ItemToColorize = (this.SourceType == MediaSourceEnum.Content) ? this.MediaSource.NodeGuid : this.MediaSource.AttachmentGuid;
        this.LastAttachmentGuid = this.ItemToColorize;

        ClearSelectedItemInfo();
    }


    /// <summary>
    /// Loads dialogs according user's configuration.
    /// </summary>
    private void LoadUserConfiguration()
    {
        if (CMSContext.CurrentUser.UserSettings.UserDialogsConfiguration != null)
        {
            XmlData dialogConfig = CMSContext.CurrentUser.UserSettings.UserDialogsConfiguration;

            string siteName = "";
            string aliasPath = "";
            string viewMode = "";

            // Store configuration based on the current tab
            switch (this.SourceType)
            {
                case MediaSourceEnum.Content:
                    siteName = (dialogConfig.ContainsColumn("content.sitename") ? dialogConfig["content.sitename"].ToString() : "");
                    aliasPath = (dialogConfig.ContainsColumn("content.path") ? dialogConfig["content.path"] : "");
                    viewMode = (dialogConfig.ContainsColumn("content.viewmode") ? dialogConfig["content.viewmode"].ToString() : "");
                    break;

                case MediaSourceEnum.DocumentAttachments:
                    viewMode = (dialogConfig.ContainsColumn("attachments.viewmode") ? dialogConfig["attachments.viewmode"].ToString() : "");
                    break;
            }

            // Update dialog configuration
            if (!string.IsNullOrEmpty(siteName))
            {
                this.siteSelector.SiteName = siteName;
            }

            if (!string.IsNullOrEmpty(aliasPath))
            {
                if (aliasPath.StartsWith(this.Config.ContentStartingPath.ToLower()))
                {
                    this.NodeID = GetContentNodeId(aliasPath);
                }
                else
                {
                    this.NodeID = GetContentNodeId(this.Config.ContentStartingPath);
                }

                // Initialize root node
                if (this.NodeID == 0)
                {
                    this.NodeID = GetContentNodeId("/");
                    this.mediaView.ShowParentButton = false;
                }

                // Select and expand node
                this.contentTree.NodeID = this.NodeID;
                this.contentTree.ExpandNodeID = this.NodeID;
            }
            else
            {
                // Reset selected node to root node
                this.NodeID = GetContentNodeId("/");
                this.contentTree.NodeID = this.NodeID;
                this.contentTree.ExpandNodeID = this.NodeID;
                this.mediaView.ShowParentButton = false;
            }

            if (!string.IsNullOrEmpty(viewMode))
            {
                this.menuElem.SelectedViewMode = CMSDialogHelper.GetDialogViewMode(viewMode);
            }
        }
        else
        {
            // Initialize site selector
            this.siteSelector.SiteID = CMSContext.CurrentSiteID;
            if (!CMSContext.CurrentUser.IsGlobalAdministrator)
            {
                this.siteSelector.Enabled = false;
            }
        }
    }


    /// <summary>
    /// Loads currently selected item.
    /// </summary>
    private void LoadSelectedItem()
    {
        if (CurrentAttachmentInfo != null)
        {
            SelectMediaItem(CurrentAttachmentInfo.AttachmentName, CurrentAttachmentInfo.AttachmentExtension,
                CurrentAttachmentInfo.AttachmentImageWidth, CurrentAttachmentInfo.AttachmentImageHeight, CurrentAttachmentInfo.AttachmentSize, CurrentAttachmentInfo.AttachmentUrl);
        }
    }

    #endregion


    #region "Private methods"

    /// <summary>
    /// Performs necessary permissions check.
    /// </summary>
    private void CheckPermissions()
    {
        // Check 'READ' permission for the specific document if attachments are being created
        if ((this.SourceType == MediaSourceEnum.DocumentAttachments) && (!this.AttachmentsAreTemporary))
        {
            if (CMSContext.CurrentUser.IsAuthorizedPerDocument(this.TreeNodeObj, NodePermissionsEnum.Read) != AuthorizationResultEnum.Allowed)
            {
                string errMsg = string.Format(ResHelper.GetString("cmsdesk.notauthorizedtoreaddocument"), this.TreeNodeObj.DocumentName);

                // Redirect to access denied page
                AccessDenied(errMsg);
            }
        }
    }


    /// <summary>
    /// Ensures that required data are displayed
    /// </summary>
    private void EnsureLoadedData()
    {
        bool processLoad = true;
        bool isLink = (this.Config.OutputFormat == OutputFormatEnum.BBLink || this.Config.OutputFormat == OutputFormatEnum.HTMLLink) ||
            (this.Config.OutputFormat == OutputFormatEnum.URL && this.SelectableContent == SelectableContentEnum.AllContent);

        // If all content is selctable do not select root by default - leave selection empty
        if ((this.SelectableContent == SelectableContentEnum.AllContent) && !isLink && !UrlHelper.IsPostback())
        {
            // Even no file is selected by default load source for the Attachment tab
            processLoad = (this.SourceType == MediaSourceEnum.DocumentAttachments);

            this.NodeID = 0;
            this.contentTree.NodeID = this.NodeID;
            this.Properties.ClearProperties(true);
        }

        // Clear properties if link dialog is opened and no link is edited
        if (!UrlHelper.IsPostback() && isLink && !this.IsItemLoaded)
        {
            this.Properties.ClearProperties(true);
        }

        // If no action takes place
        if ((this.CurrentAction == "") ||
            !(isLink && this.CurrentAction.Contains("edit")))
        {
            if (!this.CurrentAction.Contains("edit") && processLoad && !(!UrlHelper.IsPostback() && processLoad))
            {
                LoadDataSource();
            }

            // Select folder comming from user/selected item configuration
            if (!UrlHelper.IsPostback() && processLoad)
            {
                HandleFolderAction(this.NodeID.ToString(), false, false);
            }
        }
    }


    /// <summary>
    /// Initializes properties controls.
    /// </summary>
    private void SetupProperties()
    {
        this.htmlLinkProp.Visible = false;
        this.htmlMediaProp.Visible = false;
        this.bbLinkProp.Visible = false;
        this.bbMediaProp.Visible = false;
        this.urlProp.Visible = false;

        this.Properties.Visible = true;

        this.htmlLinkProp.StopProcessing = !this.htmlLinkProp.Visible;
        this.htmlMediaProp.StopProcessing = !this.htmlMediaProp.Visible;
        this.bbLinkProp.StopProcessing = !this.bbLinkProp.Visible;
        this.bbMediaProp.StopProcessing = !this.bbMediaProp.Visible;
        this.urlProp.StopProcessing = !this.urlProp.Visible;
        this.nodeGuidProp.StopProcessing = !this.nodeGuidProp.Visible;

        this.Properties.Config = this.Config;
    }


    /// <summary>
    /// Initializes additional controls
    /// </summary>
    private void SetupControls()
    {
        // Generate permanent URLs whenever node GUID output required        
        if (this.Config.OutputFormat != OutputFormatEnum.NodeGUID)
        {
            this.UsePermanentUrls = SettingsKeyProvider.GetBoolValue(CMSContext.CurrentSiteName + ".CMSUsePermanentURLs");
        }
        else
        {
            // Select current site and disable change
            this.siteSelector.Value = this.SiteID = CMSContext.CurrentSiteID;
            this.siteSelector.Enabled = false;
        }
        this.mediaView.UsePermanentUrls = this.UsePermanentUrls;
        // Set editor client id for properties
        this.Properties.EditorClientID = this.Config.EditorClientID;

        InitializeMenuElement();
        InitializeDesignScripts();

        this.mediaView.IsLiveSite = this.IsLiveSite;
        this.mediaView.SelectableContent = this.SelectableContent;
        this.mediaView.SourceType = this.SourceType;
        this.mediaView.ViewMode = this.menuElem.SelectedViewMode;
        this.mediaView.ResizeToHeight = this.Config.ResizeToHeight;
        this.mediaView.ResizeToMaxSideSize = this.Config.ResizeToMaxSideSize;
        this.mediaView.ResizeToWidth = this.Config.ResizeToWidth;
        this.mediaView.AtachmentNodeParentID = this.Config.AttachmentParentID;

        this.Properties.IsLiveSite = this.IsLiveSite;

        if (!IsAsyncPostback)
        {
            // Initialize scripts
            InitializeControlScripts();
        }

        // Based on the required source type perform setting of necessary controls
        if (this.SourceType == MediaSourceEnum.Content)
        {
            if (!IsAsyncPostback)
            {
                // Initialize content tree control
                InitializeContentTree();
            }
            else
            {
                this.contentTree.Visible = false;
                this.contentTree.StopProcessing = true;
            }
        }
        else
        {
            // Hide and disable content related controls
            HideContentControls();
        }
    }


    /// <summary>
    /// Initialize design jQuery scripts
    /// </summary>
    private void InitializeDesignScripts()
    {
        StringBuilder sb = new StringBuilder();
        sb.Append("setTimeout('InitializeDesign();',10);");
        sb.Append("$(window).resize(function() { InitializeDesign(); });");
        ScriptManager.RegisterStartupScript(Page, typeof(Page), "designScript", sb.ToString(), true);
    }


    /// <summary>
    /// Initializes menu element.
    /// </summary>
    private void InitializeMenuElement()
    {
        // Let child controls now what the source type is
        this.menuElem.IsLiveSite = this.IsLiveSite;
        this.menuElem.SourceType = this.SourceType;
        this.menuElem.ResizeToHeight = this.Config.ResizeToHeight;
        this.menuElem.ResizeToMaxSideSize = this.Config.ResizeToMaxSideSize;
        this.menuElem.ResizeToWidth = this.Config.ResizeToWidth;

        // Based on the required source type perform setting of necessary controls
        if (this.SourceType == MediaSourceEnum.Content)
        {
            this.menuElem.NodeID = this.NodeID;
        }
        else
        {
            // Initialize menu element for attachments
            this.menuElem.DocumentID = this.Config.AttachmentDocumentID;
            this.menuElem.FormGUID = this.Config.AttachmentFormGUID;
            this.menuElem.ParentNodeID = this.Config.AttachmentParentID;
        }
    }


    /// <summary>
    /// Initializes all the script required for communication between controls.
    /// </summary>
    private void InitializeControlScripts()
    {
        // SetAction function setting action name and passed argument
        string setAction = "function SetAction(action, argument) {                                              " +
                           "    var hdnAction = document.getElementById('" + this.hdnAction.ClientID + "');     " +
                           "    var hdnArgument = document.getElementById('" + this.hdnArgument.ClientID + "'); " +
                           @"    if ((hdnAction != null) && (hdnArgument != null)) {                             
                                   if (action != null) {                                                       
                                       hdnAction.value = action;                                               
                                   }                                                                           
                                   if (argument != null) {                                                     
                                       hdnArgument.value = argument;                                           
                                   }                                                                           
                               }                                                                               
                           }                                                                                    ";

        // Prepare for upload
        string refreshType = CMSDialogHelper.GetMediaSource(this.SourceType);
        string cmdName = (this.SourceType == MediaSourceEnum.DocumentAttachments) ? "attachment" : "content";

        string initRefresh = @" function InitRefresh_" + refreshType + "(message, fullRefresh, itemInfo, action) {      " +
                             @" if((message != null) && (message != ''))                                   
                                     {                                                                          
                                        window.alert(message);                                                  
                                     }                                                                          
                                     else                                                                       
                                     {
                                        if(action == 'insert')
                                        {
                                            SetAction('" + cmdName + "created', itemInfo);                              " +
                             @"         }
                                        else if(action == 'update')
                                        {
                                            SetAction('" + cmdName + "updated', itemInfo);                              " +
                             @"         }
                                        else if(action == 'refresh')
                                        {
                                            SetAction('" + cmdName + "edit', itemInfo);                                 " +
                             @"         }                                                                                                                                                                              
                                        RaiseHiddenPostBack();                                                  
                                     }                                                                          
                                   }                                                                                    ";

        setAction += initRefresh;

        // Get reffernce causing postback to hidden button
        string postBackRef = ControlsHelper.GetPostBackEventReference(this.hdnButton, "");
        string raiseOnAction = " function RaiseHiddenPostBack(){" + postBackRef + ";}\n";

        this.ltlScript.Text = ScriptHelper.GetScript(setAction + raiseOnAction);
    }


    /// <summary>
    /// Resolves macros used in update script according specified item information.
    /// </summary>
    /// <param name="attachment">Attachment information.</param>
    /// <param name="url">URL of the specified attachment.</param>
    /// <param name="argSet">Set of arguments holding additional information on updated item.</param>
    private string ResolveUpdateScript(AttachmentInfo attachment, string url, string argSet)
    {
        string newSelectAction = "ColorizeRow('" + attachment.AttachmentGUID + "'); SetSelectAction('" + argSet + ";" + url + "'); return false;";
        string newEditAction = "EditImage(\"AttachmentGUID=" + attachment.AttachmentGUID.ToString() +
            ((attachment.AttachmentLastHistoryID > 0) ? "&versionHistoryId=" + attachment.AttachmentLastHistoryID.ToString() : "") + "\"); return false;";
        string customJavaScript = "btnEdit.attr('onclick', '').unbind('click').bind('click', function () { " +
            "if($(this).hasClass('Edited')) { " + newEditAction + " } else { $(this).attr('class', '@EDITCLASS@'); return false; } } ); " +
            "if($('body').hasClass('Gecko') || $('body').hasClass('Gecko3')) { btnEdit.attr('class', '@EDITCLASS@'); };";

        string updateScript = "";
        if (this.menuElem.SelectedViewMode == DialogViewModeEnum.ListView)
        {
            // Get thumbnail dimensions
            int[] dims = ImageHelper.EnsureImageDimensions(0, 0, 200, attachment.AttachmentImageWidth, attachment.AttachmentImageHeight);

            customJavaScript = "var btnEdit = $(\"#@GUID@ input[id$='action_edit']\");" + customJavaScript.Replace("@EDITCLASS@", "UnigridActionButton Edited");

            updateScript = UPDT_SCRIPT_LIST;
            updateScript = updateScript.Replace("@THUMBWIDTH@", dims[0].ToString()).Replace("@THUMBHEIGHT@", dims[1].ToString());
        }
        else if (this.menuElem.SelectedViewMode == DialogViewModeEnum.TilesView)
        {
            // Get thumbnail dimensions
            int[] dims = ImageHelper.EnsureImageDimensions(0, 0, 200, attachment.AttachmentImageWidth, attachment.AttachmentImageHeight);

            customJavaScript = "var btnEdit = $(\"#@GUID@ input[id$='btnEdit']\");" + customJavaScript.Replace("@EDITCLASS@", "Edited");

            updateScript = UPDT_SCRIPT_TILES;
            updateScript = updateScript.Replace("@THUMBWIDTH@", dims[0].ToString()).Replace("@THUMBHEIGHT@", dims[1].ToString());
            updateScript = updateScript.Replace("@MAXSIDESIZE@", "48");
        }
        else
        {
            int[] dims = CMSDialogHelper.GetThumbImageDimensions(attachment.AttachmentImageHeight, attachment.AttachmentImageWidth, 95, 160);

            customJavaScript = "var btnEdit = $(\"#@GUID@ input[id$='btnEdit']\");" + customJavaScript.Replace("@EDITCLASS@", "Edited");

            updateScript = UPDT_SCRIPT_THUMBS;
            updateScript = updateScript.Replace("@WIDTH@", dims[1].ToString()).Replace("@HEIGHT@", dims[0].ToString());
        }
        updateScript = updateScript.Replace("@CUSTOMSCRIPT@", customJavaScript);
        updateScript = updateScript.Replace("@GUID@", attachment.AttachmentGUID.ToString()).Replace("@ACTION@", newSelectAction).Replace("@NEWGUID@", Guid.NewGuid().ToString());

        // Update List & Tiles view specific information
        if ((this.menuElem.SelectedViewMode == DialogViewModeEnum.ListView) || (this.menuElem.SelectedViewMode == DialogViewModeEnum.TilesView))
        {
            // Update size after conversion
            long size = ValidationHelper.GetLong(attachment.AttachmentSize, 0);
            updateScript = updateScript.Replace("@NEWSIZE@", DataHelper.GetSizeString(size)).Replace("@NEWEXT@", attachment.AttachmentExtension.Trim('.'));
        }

        return updateScript;
    }


    /// <summary>
    /// Loads all files for the view control.
    /// </summary>
    private void LoadDataSource()
    {
        if (this.SourceType == MediaSourceEnum.Content)
        {
            LoadContentDataSource(this.LastSearchedValue);
        }
        else
        {
            LoadAttachmentsDataSource(this.LastSearchedValue);
        }
    }


    /// <summary>
    /// Performs actions necessary to select particular item from a list.
    /// </summary>
    private void SelectMediaItem(string argument)
    {
        if (!string.IsNullOrEmpty(argument))
        {
            string[] argArr = argument.ToString().Split(';');
            if (argArr.Length >= 2)
            {
                // Get information from argument
                string name = argArr[0];
                string ext = argArr[1];
                int imageWidth = ValidationHelper.GetInteger(argArr[2], 0);
                int imageHeight = ValidationHelper.GetInteger(argArr[3], 0);
                long size = ValidationHelper.GetLong(argArr[argArr.Length - 2], 0);
                string url = argArr[argArr.Length - 1];

                // Do not update properties when selecting recently edited image item
                bool avoidPropUpdate = false;

                // Remember last selected attachment GUID
                if (this.SourceType == MediaSourceEnum.DocumentAttachments)
                {
                    Guid attGuid = ValidationHelper.GetGuid(argArr[4], Guid.Empty);

                    avoidPropUpdate = (this.LastAttachmentGuid == attGuid);

                    this.LastAttachmentGuid = attGuid;
                    this.ItemToColorize = this.LastAttachmentGuid;
                }
                else
                {
                    Guid nodeAttGuid = ValidationHelper.GetGuid(argArr[10], Guid.Empty);
                    this.Properties.SiteDomainName = this.mediaView.SiteObj.DomainName;

                    avoidPropUpdate = (this.ItemToColorize == nodeAttGuid);

                    this.ItemToColorize = nodeAttGuid;
                    if (this.ItemToColorize == Guid.Empty)
                    {
                        this.ItemToColorize = ValidationHelper.GetGuid(argArr[6], Guid.Empty);
                    }
                }

                avoidPropUpdate = (avoidPropUpdate && this.IsEditImage);
                if (!avoidPropUpdate)
                {
                    // Select item
                    base.SelectMediaItem(name, ext, imageWidth, imageHeight, size, url);
                }
            }
        }
    }


    /// <summary>
    /// Performs actions necessary to select particular item from a list.
    /// </summary>
    private void SelectMediaItem(string docName, string url)
    {
        SelectMediaItem(docName, null, 0, 0, 0, url);
    }


    /// <summary>
    /// Clears hidden control elements fo future use.
    /// </summary>
    private void ClearActionElems()
    {
        this.CurrentAction = "";
        this.hdnArgument.Value = "";
    }


    /// <summary>
    /// Displays properties in full size.
    /// </summary>
    private void DisplayFull()
    {
        if (this.divDialogView.Attributes["class"] == "DialogViewContent")
        {
            this.divDialogView.Attributes["class"] = "DialogElementHidden";
            this.divDialogResizer.Attributes["class"] = "DialogElementHidden";
            this.divDialogProperties.Attributes["class"] = "DialogPropertiesFullSize";

            if (IsFullDisplay)
            {
                this.pnlUpdateView.Update();
                this.pnlUpdateProperties.Update();
            }
            else
            {
                this.pnlUpdateContent.Update();
                IsFullDisplay = true;
            }
        }
    }


    /// <summary>
    /// Displays properties in default size.
    /// </summary>
    private void DisplayNormal()
    {
        if (this.divDialogView.Attributes["class"] == "DialogElementHidden")
        {
            this.divDialogView.Attributes["class"] = "DialogViewContent";
            this.divDialogResizer.Attributes["class"] = "DialogResizerV";
            this.divDialogProperties.Attributes["class"] = "DialogProperties";

            if (IsFullDisplay)
            {
                this.pnlUpdateContent.Update();
                IsFullDisplay = false;
            }
            else
            {
                this.pnlUpdateView.Update();
                this.pnlUpdateProperties.Update();
            }
        }
    }

    #endregion


    #region "Content methods"

    /// <summary>
    /// Hides and disables content related controls.
    /// </summary>
    private void HideContentControls()
    {
        this.pnlLeftContent.Visible = false;
        this.plcSeparator.Visible = false;
        this.pnlRightContent.CssClass = "DialogCompleteBlock";
        this.siteSelector.StopProcessing = true;
        this.contentTree.StopProcessing = true;
    }


    /// <summary>
    /// Initializes content tree element.
    /// </summary>
    private void InitializeContentTree()
    {
        this.contentTree.Visible = true;

        this.contentTree.DeniedNodePostback = false;
        this.contentTree.NodeTextTemplate = "<span class=\"ContentTreeItem\" onclick=\"SelectNode(##NODEID##, this); SetAction('contentselect', '##NODEID##;##NODECHILDNODESCOUNT##'); RaiseHiddenPostBack(); return false;\">##ICON##<span class=\"Name\">##NODENAME##</span></span>";
        this.contentTree.SelectedNodeTextTemplate = "<span id=\"treeSelectedNode\" class=\"ContentTreeSelectedItem\" onclick=\"SelectNode(##NODEID##, this); SetAction('contentselect', '##NODEID##;##NODECHILDNODESCOUNT##'); RaiseHiddenPostBack(); return false;\">##ICON##<span class=\"Name\">##NODENAME##</span></span>";
        this.contentTree.MaxTreeNodeText = "<span class=\"ContentTreeItem\" onclick=\"SetAction('morecontentselect', ##PARENTNODEID##); RaiseHiddenPostBack(); return false;\"><span class=\"Name\" style=\"font-style: italic;\">" + ResHelper.GetString("ContentTree.SeeListing") + "</span></span>";
        this.contentTree.IsLiveSite = this.IsLiveSite;
        this.contentTree.SelectOnlyPublished = this.IsLiveSite;
        this.contentTree.SelectPublishedData = this.IsLiveSite;

        this.contentTree.SiteName = this.SiteName;

        // Starting path node ID
        this.StartingPathNodeID = GetStartNodeId();
    }


    /// <summary>
    /// Returns ID of the starting node according current starting path settings.
    /// </summary>
    private int GetStartNodeId()
    {
        if (!string.IsNullOrEmpty(this.Config.ContentStartingPath))
        {
            this.contentTree.Path = this.Config.ContentStartingPath;
        }
        else if (!string.IsNullOrEmpty(CMSContext.CurrentUser.UserStartingAliasPath))
        {
            this.contentTree.Path = CMSContext.CurrentUser.UserStartingAliasPath;
        }
        return GetContentNodeId(this.contentTree.Path);
    }


    /// <summary>
    /// Returns path of the node specified by its ID.
    /// </summary>
    /// <param name="nodeId">ID of the node.</param>
    private int GetParentNodeID(int nodeId)
    {
        if (nodeId > 0)
        {
            TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
            if (tree != null)
            {
                // Get node and return its alias path
                TreeNode node = tree.SelectSingleNode(nodeId);
                if (node != null)
                {
                    return node.NodeParentID;
                }
            }
        }

        return 0;
    }

    /// <summary>
    /// Returns path of the node specified by its ID.
    /// </summary>
    /// <param name="nodeId">ID of the node.</param>
    private string GetContentPath(int nodeId)
    {
        if (nodeId > 0)
        {
            TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
            if (tree != null)
            {
                // Get node and return its alias path
                TreeNode node = tree.SelectSingleNode(nodeId);
                if (node != null)
                {
                    return ((node.NodeChildNodesCount == 0) ? TreePathUtils.GetParentPath(node.NodeAliasPath) : node.NodeAliasPath).ToLower();
                }
            }
        }

        return string.Empty;
    }


    /// <summary>
    /// Returns ID of the content node specified by its alias path.
    /// </summary>
    /// <param name="aliasPath">Alias path of the node.</param>
    private int GetContentNodeId(string aliasPath)
    {
        if (!string.IsNullOrEmpty(aliasPath))
        {
            TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
            if (tree != null)
            {
                // Return node's ID
                TreeNode node = tree.SelectSingleNode(this.SiteName, aliasPath, CMSContext.PreferredCultureCode);
                if (node != null)
                {
                    return node.NodeID;
                }
            }
        }

        return 0;
    }


    /// <summary>
    /// Applies loaded nodes to the view control.
    /// </summary>
    /// <param name="nodes">Nodes to load.</param>
    private void LoadNodes(DataSet nodes)
    {
        bool originalNotEmpty = !DataHelper.DataSourceIsEmpty(nodes);
        if (!DataHelper.DataSourceIsEmpty(nodes))
        {
            mediaView.DataSource = nodes;
        }
        else if (originalNotEmpty && IsLiveSite)
        {
            this.mediaView.InfoText = ResHelper.GetString("dialogs.document.NotAuthorizedToViewAny");
        }
    }


    /// <summary>
    /// Gets all child nodes in the specified parent path.
    /// </summary>
    /// <param name="searchText">Text to filter searched nodes.</param>
    /// <param name="tree">Tree provider used to obtain nodes.</param>
    /// <param name="parentAliasPath">Alias path of the parent.</param>
    /// <param name="siteName">Name of the related site.</param>
    private DataSet GetNodes(string searchText, TreeProvider tree, string parentAliasPath, string siteName)
    {
        // Create WHERE condition
        string where = "(NodeAliasPath <> '/')";
        if (!string.IsNullOrEmpty(searchText))
        {
            string searchTextSafe = searchText.Replace("'", "''");
            where += " AND ((AttachmentName LIKE N'%" + searchTextSafe + "%') OR (DocumentName LIKE N'%" + searchTextSafe + "%'))";
        }

        // If not all content is selectable and no additional content being displayed
        if ((this.SelectableContent != SelectableContentEnum.AllContent) && !this.IsDisplayMore)
        {
            where += " AND (ClassName = 'CMS.File')";
        }

        string columns = "ClassDisplayName, AttachmentName, AttachmentExtension, AttachmentImageWidth, AttachmentImageHeight, NodeSiteID," +
            " SiteName, NodeGUID, DocumentUrlPath, NodeAlias, NodeAliasPath, AttachmentGUID, AttachmentID, DocumentName, AttachmentSize, NodeClassID, DocumentModifiedWhen, NodeACLID, NodeChildNodesCount, DocumentCheckedOutVersionHistoryID";

        int topN = this.mediaView.CurrentTopN;

        // Get files
        tree.SelectQueryName = "selectattachments";
        DataSet nodes = null;
        if (this.IsLiveSite)
        {
            // Get published files
            columns = TreeProvider.SELECTNODES_REQUIRED_COLUMNS + ", " + columns;
            nodes = tree.SelectNodes(siteName, parentAliasPath.TrimEnd('/') + "/%", TreeProvider.ALL_CULTURES, true, null, where, "DocumentName", 1, true, topN, columns);
        }
        else
        {
            // Get latest files
            columns = DocumentHelper.GETDOCUMENTS_REQUIRED_COLUMNS + ", " + columns;
            nodes = DocumentHelper.GetDocuments(siteName, parentAliasPath.TrimEnd('/') + "/%", TreeProvider.ALL_CULTURES, true, null, where, "DocumentName", 1, false, topN, columns, tree);
        }

        // Check permissions
        return TreeSecurityProvider.FilterDataSetByPermissions(nodes, NodePermissionsEnum.Read, CMSContext.CurrentUser);
    }


    /// <summary>
    /// Loads all files for the view control.
    /// </summary>
    /// <param name="searchText">Text to filter loaded files.</param>
    private void LoadContentDataSource(string searchText)
    {
        DataSet nodes = null;

        // Load data
        if (this.NodeID > 0)
        {
            // Get selected node            
            TreeNode node = this.TreeNodeObj;
            if ((this.TreeNodeObj != null) && !(this.Config.OutputFormat == OutputFormatEnum.NodeGUID && node.NodeSiteID != CMSContext.CurrentSiteID))
            {
                // Get selected node site info
                SiteInfo si = SiteInfoProvider.GetSiteInfo(node.NodeSiteID);
                if (si != null)
                {
                    // List view
                    if (node.NodeClassName.ToLower() != "cms.file")
                    {
                        // Check permissions
                        if (CMSContext.CurrentUser.IsAuthorizedPerDocument(node, NodePermissionsEnum.ExploreTree) == AuthorizationResultEnum.Allowed || !IsLiveSite)
                        {
                            TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
                            nodes = GetNodes(searchText, tree, node.NodeAliasPath, si.SiteName);

                            LoadNodes(nodes);

                            // If all content selectable
                            bool selectableAll = this.SelectableContent == SelectableContentEnum.AllContent;
                            if (selectableAll && !this.IsDisplayMore && (this.IsAction || !UrlHelper.IsPostback()))
                            {
                                string url = this.mediaView.GetContentItemUrl(node.NodeGUID, node.DocumentUrlPath, node.NodeAlias,
                                    node.NodeAliasPath, 0, 0, 0, true);

                                this.ItemToColorize = node.NodeGUID;

                                SelectMediaItem(node.DocumentName, url);
                            }

                            // Display full-size properties if detailed view required
                            if (selectableAll && (node.NodeChildNodesCount == 0))
                            {
                                DisplayFull();
                            }
                            else
                            {
                                DisplayNormal();
                            }
                        }
                        else
                        {
                            this.mediaView.InfoText = ResHelper.GetString("dialogs.document.NotAuthorizedToExpolore");
                        }
                    }
                    else
                    {
                        // Check permissions
                        if ((CMSContext.CurrentUser.IsAuthorizedPerDocument(node, NodePermissionsEnum.Read) == AuthorizationResultEnum.Allowed))
                        {
                            // Get attachment info and initialize displayed attachment properties
                            Guid attachmentGUID = ValidationHelper.GetGuid(node.GetValue("FileAttachment"), Guid.Empty);
                            if (attachmentGUID != Guid.Empty)
                            {
                                // Get the attachment
                                TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
                                AttachmentInfo atInfo = DocumentHelper.GetAttachment(node, attachmentGUID, tree);
                                if (atInfo != null)
                                {
                                    // Get the data
                                    string extension = atInfo.AttachmentExtension;
                                    bool isContentFile = (node.NodeClassName.ToLower() == "cms.file");

                                    if (CMSDialogHelper.IsItemSelectable(this.SelectableContent, extension, isContentFile))
                                    {
                                        // Set 'get file path'
                                        atInfo.AttachmentUrl = this.mediaView.GetContentItemUrl(node.NodeGUID, node.DocumentUrlPath, node.NodeAlias, node.NodeAliasPath, 0, 0, 0, false);

                                        CurrentAttachmentInfo = atInfo;

                                        if (node.NodeChildNodesCount == 0)
                                        {
                                            // Display properties in full size
                                            DisplayFull();
                                        }
                                        else
                                        {
                                            // Load child nodes                                            
                                            nodes = GetNodes(searchText, tree, node.NodeAliasPath, si.SiteName);

                                            LoadNodes(nodes);

                                            DisplayNormal();
                                        }
                                    }
                                    else
                                    {
                                        this.mediaView.InfoText = ResHelper.GetString("dialogs.item.notselectable");

                                        DisplayNormal();
                                    }
                                }
                            }
                            else
                            {
                                DisplayNormal();
                            }
                        }
                        else
                        {
                            DisplayNormal();

                            this.mediaView.InfoText = ResHelper.GetString("dialogs.document.NotAuthorizedToViewNode");
                        }
                    }
                }
            }
        }

        this.mediaView.DataSource = nodes;
    }


    /// <summary>
    /// Handles actions related to the folders.
    /// </summary>
    /// <param name="argument">Argument related to the folder action.</param>
    /// <param name="isNewFolder">Indicates if is new folder</param>
    private void HandleFolderAction(string argument, bool isNewFolder)
    {
        HandleFolderAction(argument, isNewFolder, true);
    }


    /// <summary>
    /// Handles actions related to the folders.
    /// </summary>
    /// <param name="argument">Argument related to the folder action.</param>
    /// <param name="isNewFolder">Indicates if is new folder</param>
    /// <param name="callSelection">Indicates if selection should be called</param>
    private void HandleFolderAction(string argument, bool isNewFolder, bool callSelection)
    {
        this.NodeID = ValidationHelper.GetInteger(argument, 0);

        // Update new folder information
        this.menuElem.NodeID = this.NodeID;
        this.menuElem.UpdateActionsMenu();

        // Reload content tree if new folder was created
        if (isNewFolder)
        {
            InitializeContentTree();

            // Fill with new info
            this.contentTree.NodeID = this.NodeID;
            this.contentTree.ExpandNodeID = this.NodeID;

            this.contentTree.ReloadData();
            this.pnlUpdateTree.Update();

            ScriptManager.RegisterStartupScript(this.Page, typeof(Page), "EnsureTopWindow", "if (self.focus) { self.focus(); }", true);
        }

        // Load new data 
        LoadDataSource();

        // Load selected item
        if (CurrentAttachmentInfo != null)
        {
            string fileName = Path.GetFileNameWithoutExtension(CurrentAttachmentInfo.AttachmentName);

            if (callSelection)
            {
                SelectMediaItem(fileName, CurrentAttachmentInfo.AttachmentExtension, CurrentAttachmentInfo.AttachmentImageWidth,
                    CurrentAttachmentInfo.AttachmentImageHeight, CurrentAttachmentInfo.AttachmentSize, CurrentAttachmentInfo.AttachmentUrl);
            }

            this.ItemToColorize = this.CurrentAttachmentInfo.AttachmentGUID;
            ColorizeRow(this.ItemToColorize.ToString());
        }
        else
        {
            ColorizeLastSelectedRow();
        }

        // Get parent node ID info
        int parentId = 0;
        if (this.StartingPathNodeID != this.NodeID)
        {
            parentId = GetParentNodeID(this.NodeID);
        }
        this.mediaView.ShowParentButton = (parentId > 0);
        this.mediaView.NodeParentID = parentId;

        // Reload view control's content
        this.mediaView.Reload();
        this.pnlUpdateView.Update();

        ClearActionElems();
    }


    /// <summary>
    /// Handles actions occuring when new content (CMS.File) document was created.
    /// </summary>
    /// <param name="argument">Argument holding information on new document node ID.</param>
    private void HandleContentFileCreatedAction(string argument)
    {
        string[] argArr = argument.Split(';');
        if (argArr.Length == 1)
        {
            HandleFolderAction(argArr[0], true);
        }
    }


    /// <summary>
    /// Handles attachment edit action.
    /// </summary>
    /// <param name="argument">Attachment GUID comming from view control.</param>
    private void HandleContentEdit(string argument)
    {
        this.IsEditImage = true;

        if (!string.IsNullOrEmpty(argument))
        {
            string[] argArr = argument.Split(';');

            Guid attachmentGuid = ValidationHelper.GetGuid(argArr[0], Guid.Empty);

            // Node ID was specified
            int nodeId = 0;
            if (argArr.Length == 2)
            {
                nodeId = ValidationHelper.GetInteger(argArr[1], 0);
            }

            AttachmentInfo ai = AttachmentManager.GetAttachmentInfo(attachmentGuid, this.SiteName);
            if (ai != null)
            {
                // Get attachment node by ID
                TreeNode node = null;
                if (nodeId > 0)
                {
                    node = TreeHelper.SelectSingleNode(nodeId);
                }
                else
                {
                    node = TreeHelper.SelectSingleDocument(ai.AttachmentDocumentID);
                }

                if (node != null)
                {
                    // Check node site
                    if (node.NodeSiteID != CMSContext.CurrentSiteID)
                    {
                        this.mediaView.SiteObj = SiteInfoProvider.GetSiteInfo(node.NodeSiteID);
                    }

                    // Get node URL
                    string url = this.mediaView.GetContentItemUrl(node.NodeGUID, node.DocumentUrlPath, node.NodeAlias, node.NodeAliasPath, 0, 0, 0, false);

                    // Update properties if node is currently selected
                    if (attachmentGuid == this.ItemToColorize)
                    {
                        base.SelectMediaItem(ai.AttachmentName, ai.AttachmentExtension, ai.AttachmentImageWidth, ai.AttachmentImageHeight, ai.AttachmentSize, url);
                    }

                    // Update select action to reflect changes made during editing
                    UpdateContentSelectScript(ai, node, url);
                }
            }
        }

        ClearActionElems();
    }


    /// <summary>
    /// Updates JavaScript used to cause asynchronous post-back when item is selected in view.
    /// </summary>
    /// <param name="att">Updated content file.</param>
    /// <param name="url">URL of updated content file.</param>
    private void UpdateContentSelectScript(AttachmentInfo att, TreeNode node, string url)
    {
        string argSet = node.DocumentName + ";" + att.AttachmentExtension + ";" + att.AttachmentImageWidth + ";" + att.AttachmentImageHeight +
                        ";" + node.NodeSiteID + ";" + this.mediaView.SiteObj.SiteName + ";" + node.NodeGUID + ";" + node.DocumentUrlPath +
                        ";" + node.NodeAlias + ";" + node.NodeAliasPath + ";" + att.AttachmentGUID + ";" + att.AttachmentSize;

        string updateScript = ResolveUpdateScript(att, url, argSet);

        ScriptManager.RegisterStartupScript(this.Page, typeof(Page), "UpdateSelectAction", updateScript, true);
    }

    #endregion


    #region "Attachment methods"

    /// <summary>
    /// Loads all attachments for the view control.
    /// </summary>
    /// <param name="searchText">Text to filter loaded files.</param>
    private void LoadAttachmentsDataSource(string searchText)
    {
        DataSet attachments = null;

        int topN = this.mediaView.CurrentTopN;

        // Only unsorted attachments are being displayed
        string where = "(AttachmentIsUnsorted = 1)";

        if (!string.IsNullOrEmpty(searchText))
        {
            where += " AND (AttachmentName LIKE N'%" + searchText.Replace("'", "''") + "%')";
        }

        // Get document attachments
        if (this.Config.AttachmentDocumentID != 0)
        {
            if (this.TreeNodeObj != null)
            {
                // Check permissions
                if (CMSContext.CurrentUser.IsAuthorizedPerDocument(this.TreeNodeObj, NodePermissionsEnum.Read) == AuthorizationResultEnum.Allowed)
                {
                    TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);

                    attachments = DocumentHelper.GetAttachments(this.TreeNodeObj, where, "AttachmentOrder, AttachmentName", false, tree, topN);
                }
            }
        }
        // Get form attachments 
        else if (this.AttachmentsAreTemporary)
        {
            where += " AND (AttachmentFormGUID = '" + this.Config.AttachmentFormGUID + "')";

            attachments = this.AttachmentManager.GetAttachments(where, "AttachmentOrder, AttachmentName", false, topN);
        }

        this.mediaView.DataSource = attachments;
    }


    /// <summary>
    /// Checks attachment permissions.
    /// </summary>
    private string CheckAttachmentPermissions()
    {
        string message = "";

        // For new document
        if (this.Config.AttachmentFormGUID != Guid.Empty)
        {
            if (this.Config.AttachmentParentID == 0)
            {
                message = "Node parent node ID has to be set.";
            }

            if (!RaiseOnCheckPermissions("Create", this))
            {
                if (!CMSContext.CurrentUser.IsAuthorizedToCreateNewDocument(this.Config.AttachmentParentID, "CMS.File"))
                {
                    message = ResHelper.GetString("attach.actiondenied");
                }
            }
        }
        // For existing document
        else if (this.Config.AttachmentDocumentID > 0)
        {
            // Get document
            TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
            TreeNode node = DocumentHelper.GetDocument(this.Config.AttachmentDocumentID, tree);
            if (node == null)
            {
                message = "Given document doesn't exist!";
            }

            if (!RaiseOnCheckPermissions("Modify", this))
            {
                if (CMSContext.CurrentUser.IsAuthorizedPerDocument(node, NodePermissionsEnum.Modify) != AuthorizationResultEnum.Allowed)
                {
                    message = ResHelper.GetString("attach.actiondenied");
                }
            }
        }

        return message;
    }


    /// <summary>
    /// Handles new attachment create action.
    /// </summary>
    /// <param name="argument">Argument comming from upload control.</param>
    private void HandleAttachmentCreatedAction(string argument)
    {
        HandleAttachmentAction(argument, false);

        // Reload view
        LoadDataSource();

        this.mediaView.Reload();
        this.pnlUpdateView.Update();
    }


    /// <summary>
    /// Handles attachment update action.
    /// </summary>
    /// <param name="argument">Argument comming from upload control.</param>
    private void HandleAttachmentUpdatedAction(string argument)
    {
        HandleAttachmentAction(argument, true);
    }


    /// <summary>
    /// Handles attachment edit action.
    /// </summary>
    /// <param name="argument">Attachment GUID comming from view control.</param>
    private void HandleAttachmentEdit(string argument)
    {
        this.IsEditImage = true;

        if (!string.IsNullOrEmpty(argument))
        {
            string[] argArr = argument.Split(';');

            Guid attachmentGuid = ValidationHelper.GetGuid(argArr[0], Guid.Empty);

            AttachmentInfo ai = null;

            int versionHistoryId = ValidationHelper.GetInteger(this.TreeNodeObj.GetValue("DocumentCheckedOutversionHistoryID"), 0);
            if (versionHistoryId == 0)
            {
                ai = AttachmentManager.GetAttachmentInfo(attachmentGuid, CMSContext.CurrentSiteName);
            }
            else
            {
                VersionManager vm = new VersionManager(this.TreeNodeObj.TreeProvider);
                if (vm != null)
                {
                    IDataClass dc = vm.GetAttachmentVersion(versionHistoryId, attachmentGuid);
                    if ((dc == null) || (dc.IsEmpty()))
                    {
                        ai = null;
                    }
                    else
                    {
                        ai = new AttachmentInfo(dc.DataRow, ConnectionHelper.GetConnection());
                        ai.AttachmentID = ValidationHelper.GetInteger(dc.DataRow["AttachmentHistoryID"], 0);
                    }
                    ai.AttachmentLastHistoryID = versionHistoryId;
                }
            }

            if (ai != null)
            {
                string nodeAliasPath = "";
                if (this.TreeNodeObj != null)
                {
                    nodeAliasPath = this.TreeNodeObj.NodeAliasPath;
                }

                string url = this.mediaView.GetAttachmentItemUrl(ai.AttachmentGUID, ai.AttachmentName, nodeAliasPath, ai.AttachmentImageHeight, ai.AttachmentImageWidth, 0);

                if (this.LastAttachmentGuid == attachmentGuid)
                {
                    base.SelectMediaItem(ai.AttachmentName, ai.AttachmentExtension, ai.AttachmentImageWidth, ai.AttachmentImageHeight, ai.AttachmentSize, url);
                }

                // Update select action to reflect changes made during editing
                UpdateAtachmentSelectScript(ai, url);
            }
        }

        ClearActionElems();
    }


    /// <summary>
    /// Updates JavaScript used to cause asynchronous post-back when item is selected in view.
    /// </summary>
    /// <param name="att">Updated atachment file.</param>
    /// <param name="url">URL of updated attachment file.</param>
    private void UpdateAtachmentSelectScript(AttachmentInfo att, string url)
    {
        string name = AttachmentHelper.GetFullFileName(Path.GetFileNameWithoutExtension(att.AttachmentName), att.AttachmentExtension);

        string argSet = name + ";" + att.AttachmentExtension + ";" + att.AttachmentImageWidth + ";" + att.AttachmentImageHeight +
                        ";" + att.AttachmentGUID.ToString() + ";" + att.AttachmentSiteID + ";" + att.AttachmentFormGUID +
                        ";" + att.AttachmentDocumentID + ";" + att.AttachmentSize;

        string updateScript = ResolveUpdateScript(att, url, argSet);

        ScriptManager.RegisterStartupScript(this.Page, typeof(Page), "UpdateSelectAction", updateScript, true);
    }


    /// <summary>
    /// Handles attachment action.
    /// </summary>
    /// <param name="argument">Argument comming from upload control.</param>
    private void HandleAttachmentAction(string argument, bool isUpdate)
    {
        // Get attachment URL first
        Guid attachmentGuid = ValidationHelper.GetGuid(argument, Guid.Empty);
        if (attachmentGuid != Guid.Empty)
        {
            // Get attachment info
            TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);

            // Ensure site information
            SiteInfo si = CMSContext.CurrentSite;
            if ((this.TreeNodeObj != null) && (si.SiteID != this.TreeNodeObj.NodeSiteID))
            {
                si = SiteInfoProvider.GetSiteInfo(this.TreeNodeObj.NodeSiteID);
            }

            AttachmentInfo ai = DocumentHelper.GetAttachment(attachmentGuid, tree, si.SiteName);
            if (ai != null)
            {
                string nodeAliasPath = (this.TreeNodeObj != null) ? this.TreeNodeObj.NodeAliasPath : null;

                if (CMSDialogHelper.IsItemSelectable(this.SelectableContent, ai.AttachmentExtension))
                {
                    // Get attachment URL
                    string url = this.mediaView.GetAttachmentItemUrl(ai.AttachmentGUID, ai.AttachmentName, nodeAliasPath, 0, 0, 0);

                    // Remember last selected attachment GUID
                    if (this.SourceType == MediaSourceEnum.DocumentAttachments)
                    {
                        this.LastAttachmentGuid = ai.AttachmentGUID;
                    }

                    SelectMediaItem(ai.AttachmentName, ai.AttachmentExtension, ai.AttachmentImageWidth, ai.AttachmentImageHeight, ai.AttachmentSize, url);

                    this.ItemToColorize = attachmentGuid;

                    ColorizeRow(this.ItemToColorize.ToString());
                }

                this.mediaView.InfoText = (isUpdate ? ResHelper.GetString("dialogs.attachment.updated") : ResHelper.GetString("dialogs.attachment.created"));

                this.pnlUpdateView.Update();
            }
        }

        ClearActionElems();
    }


    /// <summary>
    /// Hadnles actions occuring when attachment is moved.
    /// </summary>
    /// <param name="argument">Argument holding information on attachment being moved.</param>
    /// <param name="action">Action specifying whether the attachment is moved up/down.</param>
    private void HandleAttachmentMoveAction(string argument, string action)
    {
        // Check permissions
        string errMsg = CheckAttachmentPermissions();

        if (errMsg == "")
        {
            int attachmentId = ValidationHelper.GetInteger(argument, 0);
            if (attachmentId > 0)
            {
                // Move temporary attachement
                if (!this.AttachmentsAreTemporary)
                {
                    if (action == "attachmentmoveup")
                    {
                        DocumentHelper.MoveAttachmentUp(attachmentId, this.TreeNodeObj);
                    }
                    else
                    {
                        DocumentHelper.MoveAttachmentDown(attachmentId, this.TreeNodeObj);
                    }
                }
                else
                {
                    if (action == "attachmentmoveup")
                    {
                        this.AttachmentManager.MoveTemporaryAttachmentUp(attachmentId);
                    }
                    else
                    {
                        this.AttachmentManager.MoveTemporaryAttachmentDown(attachmentId);
                    }
                }

                // Reload data
                LoadDataSource();

                this.plcError.Visible = false;

                this.mediaView.Reload();
            }
        }
        else
        {
            // Display error
            this.lblError.Text = errMsg;
            this.plcError.Visible = true;
        }

        ColorizeLastSelectedRow();

        this.pnlUpdateView.Update();
    }


    /// <summary>
    /// Hadnles actions occuring when some attachment is being removed.
    /// </summary>
    /// <param name="argument">Argument holding information on attachment.</param>
    private void HandleDeleteAttachmentAction(string argument)
    {
        string errMsg = CheckAttachmentPermissions();

        if (errMsg == "")
        {
            Guid attachmentGuid = ValidationHelper.GetGuid(argument, Guid.Empty);
            if (attachmentGuid != Guid.Empty)
            {
                if (!this.AttachmentsAreTemporary)
                {
                    TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);

                    DocumentHelper.DeleteAttachment(this.TreeNodeObj, attachmentGuid, tree);
                }
                else
                {
                    // Delete temporary attachment
                    this.AttachmentManager.DeleteTemporaryAttachment(attachmentGuid, CMSContext.CurrentSiteName);
                }

                // Reload data
                LoadDataSource();
                this.mediaView.Reload();

                // Selected attachment was removed
                if (this.LastAttachmentGuid == attachmentGuid)
                {
                    // Reset properties
                    this.Properties.ClearProperties();
                    this.pnlUpdateProperties.Update();
                }
                else
                {
                    ColorizeLastSelectedRow();
                }
            }
        }
        else
        {
            // Display error
            this.lblError.Text = errMsg;
            this.plcError.Visible = true;
        }

        this.pnlUpdateView.Update();
    }

    #endregion


    #region "Common event methods"

    /// <summary>
    /// Handles actions occuring when some text is searched.
    /// </summary>
    /// <param name="argument">Argument holding information on searched text.</param>
    private void HandleSearchAction(string argument)
    {
        this.LastSearchedValue = argument;

        // Load new data filtered by searched text 
        LoadDataSource();

        // Reload content
        this.mediaView.Reload();
        this.pnlUpdateView.Update();

        // Keep focus in search text box
        ScriptManager.RegisterStartupScript(this.Page, typeof(Page), "SetSearchFocus", "setTimeout('SetSearchFocus();', 200);", true);
    }


    /// <summary>
    /// Handles actions occuring when some item is selected.
    /// </summary>
    /// <param name="argument">Argument holding information on selected item.</param>
    private void HandleSelectAction(string argument)
    {
        // Create new selected media item
        SelectMediaItem(argument);

        // Forget recent action
        ClearActionElems();
    }

    #endregion


    #region "Event handlers"

    /// <summary>
    /// Handler of the event occuring when site in selector has changed.
    /// </summary>
    protected void siteSelector_SelectedSiteChanged(object sender, EventArgs e)
    {
        this.IsAction = true;

        // Update information on current site 
        this.SiteID = this.siteSelector.SiteID;
        if (this.SiteID > 0)
        {
            this.mediaView.SiteObj = SiteInfoProvider.GetSiteInfo(this.SiteID);
        }

        // Reset selected node to root node
        this.NodeID = this.StartingPathNodeID = GetStartNodeId();
        if (this.NodeID == 0)
        {
            this.NodeID = GetContentNodeId("/");
            this.mediaView.ShowParentButton = false;
        }

        if (this.SelectableContent != SelectableContentEnum.AllContent)
        {
            this.contentTree.NodeID = this.NodeID;
            this.contentTree.ExpandNodeID = this.NodeID;

            // Reload media view for new site
            LoadDataSource();
        }
        else
        {
            this.mediaView.DataSource = null;
        }

        // Update information on parent node ID for new folder creation
        this.menuElem.NodeID = this.NodeID;
        this.menuElem.UpdateActionsMenu();

        // Reload content tree for new site
        this.contentTree.SiteName = this.siteSelector.SiteName;
        InitializeContentTree();
        this.pnlUpdateTree.Update();

        // Load selected item
        if (CurrentAttachmentInfo != null)
        {
            SelectMediaItem(CurrentAttachmentInfo.AttachmentName, CurrentAttachmentInfo.AttachmentExtension,
                CurrentAttachmentInfo.AttachmentImageWidth, CurrentAttachmentInfo.AttachmentImageHeight, CurrentAttachmentInfo.AttachmentSize, CurrentAttachmentInfo.AttachmentUrl);
        }

        // Setup media view
        this.mediaView.Reload();
        this.pnlUpdateView.Update();

        // Setup properties
        DisplayNormal();
        this.Properties.ClearProperties();
        this.pnlUpdateProperties.Update();
    }


    /// <summary>
    /// Behaves as mediator in communication line between control taking action and the rest of the same level controls.
    /// </summary>
    protected void hdnButton_Click(object sender, EventArgs e)
    {
        this.IsAction = true;

        switch (this.CurrentAction)
        {
            case "insertitem":
                GetSelectedItem();
                break;

            case "search":
                HandleSearchAction(this.CurrentArgument);
                break;

            case "select":
                HandleSelectAction(this.CurrentArgument);
                break;

            case "morecontentselect":
            case "contentselect":
                string[] argArr = this.CurrentArgument.Split(';');

                int childNodesCnt = 0;
                if (argArr.Length == 2)
                {
                    childNodesCnt = ValidationHelper.GetInteger(argArr[1], 0);
                }

                if ((this.Config.OutputFormat == OutputFormatEnum.BBLink) || (this.Config.OutputFormat == OutputFormatEnum.HTMLLink))
                {
                    this.CurrentAttachmentInfo = null;
                }

                // If more content is requested
                this.IsDisplayMore = (!this.IsDisplayMore ? ((this.CurrentAction == "morecontentselect") || (childNodesCnt > this.MaxTreeNodes)) : this.IsDisplayMore);
                HandleFolderAction(argArr[0], this.IsDisplayMore);
                break;

            case "parentselect":
                HandleFolderAction(this.CurrentArgument, true);
                break;

            case "contentcreated":
                HandleContentFileCreatedAction(this.CurrentArgument);
                break;

            case "closelisting":
                this.IsDisplayMore = false;
                HandleFolderAction(this.NodeID.ToString(), false);
                break;

            case "newfolder":
                HandleFolderAction(this.CurrentArgument, true);
                break;

            case "cancelfolder":
                ScriptManager.RegisterStartupScript(this.Page, typeof(Page), "EnsureTopWindow", "if (self.focus) { self.focus(); }", true);
                ClearActionElems();
                break;

            case "attachmentmoveup":
                HandleAttachmentMoveAction(this.CurrentArgument, this.CurrentAction);
                break;

            case "attachmentmovedown":
                HandleAttachmentMoveAction(this.CurrentArgument, this.CurrentAction);
                break;

            case "attachmentdelete":
                HandleDeleteAttachmentAction(this.CurrentArgument);
                break;

            case "attachmentcreated":
                HandleAttachmentCreatedAction(this.CurrentArgument);
                break;

            case "attachmentupdated":
                HandleAttachmentUpdatedAction(this.CurrentArgument);
                break;

            case "attachmentedit":
                HandleAttachmentEdit(this.CurrentArgument);
                break;

            case "contentedit":
                HandleContentEdit(this.CurrentArgument);
                break;

            default:
                ColorizeLastSelectedRow();
                this.pnlUpdateView.Update();
                break;
        }
    }

    #endregion
}
