using System;

using CMS.GlobalHelper;
using CMS.CMSHelper;
using CMS.TreeEngine;
using CMS.WorkflowEngine;
using CMS.UIControls;

using TreeNode = CMS.TreeEngine.TreeNode;

public partial class CMSDesk_Content_Properties_Attachments : CMSDeskPage
{
    #region "Variables"

    protected bool hasModifyPermission = true;
    protected TreeProvider tree = null;
    protected TreeNode node = null;
    protected int nodeId = 0;

    #endregion


    #region "Page events"

    protected void Page_Load(object sender, EventArgs e)
    {
        UIContext.PropertyTab = PropertyTabEnum.Attachments;

        int nodeId = QueryHelper.GetInteger("nodeid", 0);
        if (nodeId > 0)
        {
            tree = new TreeProvider(CMSContext.CurrentUser);
            node = tree.SelectSingleNode(nodeId);
            if (node != null)
            {
                // Check read permissions
                if (CMSContext.CurrentUser.IsAuthorizedPerTreeNode(node, NodePermissionsEnum.Read) == AuthorizationResultEnum.Denied)
                {
                    RedirectToAccessDenied(String.Format(ResHelper.GetString("cmsdesk.notauthorizedtoreaddocument"), node.NodeAliasPath));
                }
                // Check modify permissions
                else if (CMSContext.CurrentUser.IsAuthorizedPerTreeNode(node, NodePermissionsEnum.Modify) == AuthorizationResultEnum.Denied)
                {
                    hasModifyPermission = false;
                    lblInfo.Text = String.Format(ResHelper.GetString("cmsdesk.notauthorizedtoeditdocument"), node.NodeAliasPath);
                }
                ucAttachments.DocumentID = node.DocumentID;
                ucAttachments.NodeParentNodeID = node.NodeParentID;
                ucAttachments.NodeClassName = node.NodeClassName;

                // Resize attachment due to site settings
                string siteName = CMSContext.CurrentSiteName;
                ucAttachments.ResizeToHeight = ImageHelper.GetAutoResizeToHeight(siteName);
                ucAttachments.ResizeToWidth = ImageHelper.GetAutoResizeToWidth(siteName);
                ucAttachments.ResizeToMaxSideSize = ImageHelper.GetAutoResizeToMaxSideSize(siteName);

                // Get the node workflow
                WorkflowManager wm = new WorkflowManager(node.TreeProvider);
                WorkflowInfo wi = wm.GetNodeWorkflow(node);
                if (wi != null)
                {
                    // Ensure the document version
                    VersionManager vm = new VersionManager(node.TreeProvider);
                    ucAttachments.VersionHistoryID = vm.EnsureVersion(node, node.IsPublished);
                }
            }
        }
    }


    protected override void OnPreRender(EventArgs e)
    {
        base.OnPreRender(e);

        ucAttachments.Enabled = (menuElem.AllowEdit && hasModifyPermission && menuElem.AllowSave);
        lblInfo.Text = menuElem.WorkflowInfo;
        pnlAttachments.Visible = (lblInfo.Text != string.Empty);
    }

    #endregion
}
