﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Data;

using CMS.UIControls;
using CMS.GlobalHelper;
using CMS.MediaLibrary;
using CMS.CMSHelper;
using CMS.SettingsProvider;
using CMS.SiteProvider;
using CMS.Controls;
using CMS.ExtendedControls;


public partial class CMSModules_MediaLibrary_Controls_Dialogs_MediaView : MediaView
{
    #region "Events & delegates"
    
    /// <summary>
    /// Delegate for the event fired when UniGrid's page changed.
    /// </summary>
    public delegate void OnListReloadRequired();
    
    /// <summary>
    /// Event fired when UniGrid's page changed
    /// </summary>
    public event OnListReloadRequired ListReloadRequired;

    #endregion


    #region "Private variables"

    // Media library variables
    private int mLibraryId = 0;
    private MediaLibraryInfo mLibraryInfo = null;   
    private SiteInfo mLibrarySite = null;
    
    #endregion


    #region "Public properties"

    /// <summary>
    /// Gets or sets an ID of the media library.
    /// </summary>
    public int LibraryID
    {
        get
        {
            return this.mLibraryId;
        }
        set
        {
            this.mLibraryId = value;
        }
    }


    /// <summary>
    /// Current media library information.
    /// </summary>
    public MediaLibraryInfo LibraryInfo 
    {
        get 
        {
            if (this.mLibraryInfo == null) 
            {
                this.mLibraryInfo = MediaLibraryInfoProvider.GetMediaLibraryInfo(this.LibraryID);
            }
            return this.mLibraryInfo;
        }
        set 
        {
            this.mLibraryInfo = value;
        }
    }


    /// <summary>
    /// Indicates whether control processing should be stopped.
    /// </summary>
    public override bool StopProcessing
    {
        get
        {
            return base.StopProcessing;
        }
        set
        {
            this.innermedia.StopProcessing = value;
            base.StopProcessing = value;
        }
    }


    /// <summary>
    /// Gets currently selected page size.
    /// </summary>
    public int CurrentTopN
    {
        get
        {
            return this.innermedia.CurrentTopN;
        }
    }


    /// <summary>
    /// Gets or sets the OutputFormat (needed for correct dialog type reckognition)
    /// </summary>
    public OutputFormatEnum OutputFormat
    {
        get
        {
            return this.innermedia.OutputFormat;
        }
        set
        {
            this.innermedia.OutputFormat = value;
        }
    }

    #endregion


    #region "Private properties"

    /// <summary>
    /// Gets site info current library is related to.
    /// </summary>
    private SiteInfo LibrarySite 
    {
        get 
        {
            if ((this.mLibrarySite == null) && (this.LibraryInfo != null))
            {
                this.mLibrarySite = SiteInfoProvider.GetSiteInfo(this.LibraryInfo.LibrarySiteID);
            }            
            return this.mLibrarySite;
        }
    }

    #endregion
   

    protected void Page_Load(object sender, EventArgs e)
    {
        this.Visible = !this.StopProcessing;

        // If processing the request should not continue
        if (!this.StopProcessing)
        {
            // Initialize controls
            SetupControls();
        }

        this.innermedia.ViewMode = this.ViewMode;
        this.innermedia.IsLiveSite = this.IsLiveSite;
    }

    
    /// <summary>
    /// Loads control's content.
    /// </summary>
    /// <param name="forceSetup">Indicates whether the inner controls should be re-setuped.</param>
    public void Reload(bool forceSetup)
    {
        this.Visible = !this.StopProcessing;
        if (this.Visible)
        {
            // Initialize controls
            SetupControls();

            ReloadData(forceSetup);
        }
    }


    /// <summary>
    /// Loads control's content.
    /// </summary>    
    public void Reload()
    {
        Reload(false);
    }


    #region "Private methods"

    /// <summary>
    /// Initializes all nested controls.
    /// </summary>
    private void SetupControls()
    {
        InitializeControlScripts();

        // Initialize inner view control
        this.innermedia.ViewMode = this.ViewMode;
        this.innermedia.DataSource = this.DataSource;
        this.innermedia.SelectableContent = this.SelectableContent;
        this.innermedia.ListViewControl.GridName = "~/CMSModules/MediaLibrary/Controls/Dialogs/MediaListView.xml";
        this.innermedia.ListViewControl.OnBeforePageIndexChanging += new GridViewPageEventHandler(ListViewControl_OnBeforePageIndexChanging);
        this.innermedia.SourceType = this.SourceType;

        this.innermedia.ResizeToHeight = this.ResizeToHeight;
        this.innermedia.ResizeToMaxSideSize = this.ResizeToMaxSideSize;
        this.innermedia.ResizeToWidth = this.ResizeToWidth;

        // Set inner control binding columns
        this.innermedia.FileIdColumn = "FileGUID";
        this.innermedia.FileNameColumn = "FileName";
        this.innermedia.FileExtensionColumn = "FileExtension";
        this.innermedia.FileSizeColumn = "FileSize";
        this.innermedia.FileWidthColumn = "FileImageWidth";
        this.innermedia.FileHeightColumn = "FileImageHeight";
        
        // Register for inner media events
        this.innermedia.GetArgumentSet += new CMSAdminControls_Dialogs_LinkMediaSelector_InnerMediaView.OnGetArgumentSet(innermedia_GetArgumentSet);
        this.innermedia.GetListItemUrl += new CMSAdminControls_Dialogs_LinkMediaSelector_InnerMediaView.OnGetListItemUrl(innermedia_GetListItemUrl);
        this.innermedia.GetTilesThumbsItemUrl += new CMSAdminControls_Dialogs_LinkMediaSelector_InnerMediaView.OnGetTilesThumbsItemUrl(innermedia_GetTilesThumbsItemUrl);
    }

    
    void ListViewControl_OnBeforePageIndexChanging(object sender, GridViewPageEventArgs e)
    {
        if (this.ListReloadRequired != null) 
        {
            this.ListReloadRequired();
        }
    }


    /// <summary>
    /// Initializes scrips used by the control.
    /// </summary>
    private void InitializeControlScripts()
    {
        string script = @" function SetSelectAction(argument) {
                                // Raise select action
                                SetAction('select', argument);
                                RaiseHiddenPostBack();
                            }";

        ScriptManager.RegisterStartupScript(this, this.GetType(), "DialogsSelectAction", script, true);
    }

    
    /// <summary>
    /// Loads data from data source property.
    /// </summary>
    /// <param name="forceSetup">Indicates whether the inner controls should be re-setuped.</param>
    private void ReloadData(bool forceSetup)
    {
        this.innermedia.Reload(forceSetup);
    }

    #endregion


    #region "Inner media view event handlers"

    /// <summary>
    /// Returns argument set according passed DataRow and flag indicating whether the set is obtained for selected item.
    /// </summary>
    /// <param name="dr">DataRow with all the item data.</param>
    /// <param name="isSelected">Indicates whether the set is required for an selected item.</param>
    private string innermedia_GetArgumentSet(DataRow dr)
    {
        // Return required argument set
        return GetArgumentSet(dr);
    }


    /// <summary>
    /// Returns URL of item according specified conditions.
    /// </summary>
    /// <param name="dr">Data row holding information on item.</param>
    /// <param name="isPreview">Indicates whether the URL is requested for item preview.</param>
    private string innermedia_GetListItemUrl(DataRow dr, bool isPreview, bool notAttachment)
    {
        // Get set of important information
        string arg = GetArgumentSet(dr);

        // Get URL of the list item image
        return GetItemUrl(arg, isPreview, 0, 0, 0);
    }


    /// <summary>
    /// Returns URL of item according specified conditions.
    /// </summary>
    /// <param name="dr">Data row holding information on item.</param>
    /// <param name="isPreview">Indicates whether the URL is requested for item preview.</param>
    /// <param name="maxSideSize">Specifies maximum size of the image.</param>
    private string innermedia_GetTilesThumbsItemUrl(DataRow dr, bool isPreview, int height, int width, int maxSideSize, bool notAttachment)
    {
        string url = "";

        if (this.LibraryInfo != null)
        {
            string ext = dr["FileExtension"].ToString();
            string filePath = dr["FilePath"].ToString();
            string arg = GetArgumentSet(dr);

            // If image is requested for preview
            if (isPreview)
            {
                // Check if file has a preview
                if (!ImageHelper.IsImage(ext))
                {
                    // File isn't image and no preview exists - get the default file icon
                    url = ImageHelper.GetFileIconUrl(ext, "");
                }
                else
                {
                    // Try to get preview or image itself
                    url = GetItemUrl(arg, isPreview, height, width, maxSideSize);
                }
            }
            else
            {
                url = GetItemUrl(arg, false, 0, 0, 0);
            }
        }

        return url;
    }

    #endregion


    #region "Helper methods"

    /// <summary>
    /// Returns argument set for the passed file data row.
    /// </summary>
    /// <param name="dr">Data row object holding all the data on the current file.</param>
    /// <param name="isSelectedItem">Indicates whether the argument is required for selected item initialization.</param>
    private static string GetArgumentSet(DataRow dr)
    {
        return dr["FileGUID"].ToString() + ";" + dr["FileName"].ToString() + ";" + dr["FilePath"].ToString() + ";" +
            dr["FileExtension"] + ";" + dr["FileImageWidth"].ToString() + ";" + dr["FileImageHeight"].ToString() + ";" +
            dr["FileTitle"].ToString() + ";" + dr["FileSize"].ToString();
    }


    /// <summary>
    /// Returns URL of the media item according site settings.
    /// </summary>
    /// <param name="argument">Argument containing information on current media item.</param>
    /// <param name="isPreview">Indicates whether the file has a preview file or not.</param>
    /// <param name="maxSideSize">Maximum dimension for images displayed for tile and thumbnails view.</param>
    private string GetItemUrl(string argument, bool isPreview, int height, int width, int maxSideSize)
    {
        string[] argArr = argument.Split(';');
        if (argArr.Length >= 2)
        {
            // Get information from argument
            Guid fileGuid = ValidationHelper.GetGuid(argArr[0], Guid.Empty);
            string fileName = argArr[1];
            string fileExtension = argArr[3];
            string filePath = argArr[2];

            return GetItemUrl(this.LibrarySite, fileGuid, fileName, fileExtension, filePath, isPreview, height, width, maxSideSize);
        }

        return "";
    }


    /// <summary>
    /// Returnd URL of the media item according site settings.
    /// </summary>
    /// <param name="site">Information on site file belongs to.</param>
    /// <param name="fileGuid">GUID of the file URL is generated for.</param>
    /// <param name="fileName">Name of the file URL is generated for.</param>
    /// <param name="filePath">Media file path of the file URL is generated for.</param>
    /// <param name="isPreview">Indicates whether the file URL is generated for preview.</param>
    /// <param name="maxSideSize">Maximum dimension for images displayed for tile and thumbnails view.</param>
    public string GetItemUrl(SiteInfo site, Guid fileGuid, string fileName, string fileExtension, string filePath, bool isPreview, int height, int width, int maxSideSize)
    {
        string result = "";
        bool resize = (maxSideSize > 0);

        fileName = this.UsePermanentUrls ? AttachmentHelper.GetFullFileName(fileName, fileExtension) : fileName;

        if (this.Config.UseFullURL || (CMSContext.CurrentSiteID != this.LibraryInfo.LibrarySiteID) || (CMSContext.CurrentSiteID != base.GetCurrentSiteId()))
        {
            // If permanent URLs should be generated
            if (this.UsePermanentUrls || resize || isPreview)
            {
                // URL in format 'http://domainame/getmedia/123456-25245-45454-45455-5455555545/testfile.gif'
                result = MediaFileInfoProvider.GetMediaFileAbsoluteUrl(site.SiteName, fileGuid, fileName);
            }
            else
            {
                // URL in format 'http://domainame/cms/EcommerceSite/media/testlibrary/folder1/testfile.gif'
                result = MediaFileInfoProvider.GetMediaFileUrl(site.SiteName, this.LibraryInfo.LibraryFolder, filePath);
                result = UrlHelper.GetAbsoluteUrl(result, site.DomainName, UrlHelper.GetApplicationUrl(site.DomainName), HttpContext.Current.Request.Url.AbsolutePath);
            }
        }
        else
        {
            if (this.UsePermanentUrls || resize || isPreview)
            {
                // URL in format '/cms/getmedia/123456-25245-45454-45455-5455555545/testfile.gif'
                result = MediaFileInfoProvider.GetMediaFileUrl(fileGuid, fileName);
            }
            else
            {
                // URL in format '/cms/EcommerceSite/media/testlibrary/folder1/testfile.gif'
                result = MediaFileInfoProvider.GetMediaFileUrl(CMSContext.CurrentSiteName, this.LibraryInfo.LibraryFolder, filePath);
            }            
        }

        // If image dimensions are specified
        if (resize)
        {
            result = UrlHelper.AddParameterToUrl(result, "maxsidesize", maxSideSize.ToString());
        }
        if (height > 0)
        {
            result = UrlHelper.AddParameterToUrl(result, "height", height.ToString());
        }
        if (width > 0)
        {
            result = UrlHelper.AddParameterToUrl(result, "width", width.ToString());
        }

        // Media selctor should returns non-resolved URL in all cases
        bool isMediaSelector = (this.OutputFormat == OutputFormatEnum.URL) && (this.SelectableContent == SelectableContentEnum.OnlyMedia);

        return (isMediaSelector ? result : UrlHelper.ResolveUrl(result));
    }

    #endregion 
}
