using System;
using System.Data;
using System.Web;
using System.Web.UI;

using CMS.PortalControls;
using CMS.GlobalHelper;
using CMS.CMSHelper;
using CMS.Controls;

public partial class CMSWebParts_Membership_OnlineUsers : CMSAbstractWebPart
{
    #region "Public properties"

    /// <summary>
    /// Select only top n users.
    /// </summary>
    public int SelectTopN
    {
        get
        {
            return ValidationHelper.GetInteger(GetValue("SelectTopN"), 0);
        }
        set
        {
            SetValue("SelectTopN", value);
        }
    }


    /// <summary>
    /// Select only users localized in specified path.  
    /// </summary>
    public string Path
    {
        get
        {
            return ValidationHelper.GetString(GetValue("Path"), "");
        }
        set
        {
            SetValue("Path", value);
        }
    }


    /// <summary>
    /// Gets or sets the transformation name
    /// </summary>
    public string TransformationName
    {
        get
        {
            return ValidationHelper.GetString(GetValue("TransformationName"), "");
        }
        set
        {
            SetValue("TransformationName", value);
        }
    }


    /// <summary>
    /// Gets or sets the additional info text
    /// </summary>
    public string AdditionalInfoText
    {
        get
        {
            return ValidationHelper.GetString(GetValue("AdditionalInfoText"), "");
        }
        set
        {
            SetValue("AdditionalInfoText", value);
        }
    }

    /// <summary>
    /// Gets or sets the no users on-line text
    /// </summary>
    public string NoUsersOnlineText
    {
        get
        {
            return ValidationHelper.GetString(GetValue("NoUsersOnlineText"), "");
        }
        set
        {
            SetValue("NoUsersOnlineText", value);
        }
    }

    #endregion


    #region "Events"

    /// <summary>
    /// OnPreRender override method.
    /// </summary>
    protected override void OnPreRender(EventArgs e)
    {
        base.OnPreRender(e);
        SetupControl();
    }

    #endregion


    #region "Methods"

    /// <summary>
    /// Setups control properties.
    /// </summary>
    protected void SetupControl()
    {
        // Check StopProcessing property
        if (StopProcessing)
        {
            Visible = false;
        }
        else
        {
            SetContext();

            DataSet users = null;
            bool transLoaded = false;

            // Load transformation
            if (!string.IsNullOrEmpty(TransformationName))
            {
                repUsers.ItemTemplate = CMSDataProperties.LoadTransformation(this, TransformationName, false);
                transLoaded = true;
            }

            if ((transLoaded) || (!String.IsNullOrEmpty(Path)))
            {
                // Get or generate cache item name
                string useCacheItemName = DataHelper.GetNotEmpty(CacheItemName, CMSContext.BaseCacheKey + "|" + Context.Request.Url + "|" + ClientID + "|" + "onlineUsersWebPart");


                // Try to get data from cache 
                if (CacheMinutes == 0)
                {
                    // No caching
                    users = SessionManager.GetOnlineUsers(null, null, SelectTopN, Path, CMSContext.CurrentSiteName, false, false);
                }
                else
                {
                    // Try to retrieve data from the cache
                    if (!CacheHelper.TryGetItem<DataSet>(useCacheItemName, out users))
                    {
                        users = SessionManager.GetOnlineUsers(null, null, SelectTopN, Path, CMSContext.CurrentSiteName, false, false);
                        if (!DataHelper.DataSourceIsEmpty(users))
                        {
                            CacheHelper.Add(useCacheItemName, users, null, DateTime.Now.AddMinutes(CacheMinutes), System.Web.Caching.Cache.NoSlidingExpiration);
                        }
                    }
                }

                // Data bind
                if (!DataHelper.DataSourceIsEmpty(users))
                {
                    // Set to repeater
                    repUsers.DataSource = users;
                    repUsers.DataBind();
                }
            }


            int authenticated = SessionManager.AuthenticatedUsers;
            int publicUsers = SessionManager.PublicUsers;
            if (!String.IsNullOrEmpty(Path))
            {
                authenticated = 0;
                if (!DataHelper.DataSourceIsEmpty(users))
                {
                    if (SelectTopN <= 0 || users.Tables[0].Rows.Count < SelectTopN)
                    {
                        authenticated = users.Tables[0].Rows.Count;
                    }
                    else
                    {
                        users = SessionManager.GetOnlineUsers(null, null, -1, Path, CMSContext.CurrentSiteName, false, false);
                        if (!DataHelper.DataSourceIsEmpty(users))
                        {
                            authenticated = users.Tables[0].Rows.Count;
                        }
                    }
                }

                // Get number of public users
                publicUsers = SessionManager.GetPublicUsersNumber(CMSContext.CurrentSiteName, Path);
            }

            // Check if at least one user is online
            if ((publicUsers + authenticated) == 0)
            {
                ltrAdditionaInfos.Text = NoUsersOnlineText;
            }
            else
            {
                ltrAdditionaInfos.Text = string.Format(AdditionalInfoText, publicUsers + authenticated, publicUsers, authenticated);
            }

        }

        ReleaseContext();
    }


    /// <summary>
    /// Clears the cached items
    /// </summary>
    public override void ClearCache()
    {
        string useCacheItemName = DataHelper.GetNotEmpty(CacheItemName, CMSContext.BaseCacheKey + "|" + Context.Request.Url + "|" + ClientID + "|" + "onlineUsersWebPart");

        CacheHelper.ClearCache(useCacheItemName);
    }

    #endregion
}
